@@ -126,7 +126,7 @@ sub setTsTranslations($$%)
 # Generate .ts files from .po files.
 sub generateTs()
 {
-	my @languages = ("de", "es", "fr", "ru", "it");
+	my @languages = ("de", "es", "fr", "ru", "it", "pl");
 	my $tmpdir = ".tsdir";
 	mkdir $tmpdir unless -d $tmpdir;
 	mkdir "po" unless -d "po";
@@ -147,9 +147,12 @@ sub generateTs()
 
 my $have_vorbis = 1;
 my $have_flac = 1;
+my $have_flac_picture = 1;
 my $have_id3lib = 1;
 my $have_taglib = 1;
+my $taglib_includes = "-I/usr/include/taglib";
 my $have_mp4v2 = 1;
+my $have_qtdbus = 0;
 my $have_tunepimp = 5;
 my $qmake_cmd = "qmake";
 my $enable_pch = 0;
@@ -159,6 +162,13 @@ my $bindir;
 my $datarootdir;
 my $docdir;
 my $translationsdir;
+my $extra_includes;
+my $extra_libs;
+my $extra_defines;
+my $extra_cxxflags;
+my $db2html;
+my $perl_cmd;
+my $xsl_stylesheet;
 my $from_configure;
 my $generate_ts;
 
@@ -167,12 +177,19 @@ while (my $opt = shift) {
 		$have_vorbis = 0;
   } elsif ($opt eq "--without-flac") {
 		$have_flac = 0;
+		$have_flac_picture = 0;
+  } elsif ($opt eq "--without-flac-picture") {
+		$have_flac_picture = 0;
   } elsif ($opt eq "--without-id3lib") {
 		$have_id3lib = 0;
   } elsif ($opt eq "--without-taglib") {
 		$have_taglib = 0;
+	} elsif (substr($opt, 0, 23) eq "--with-taglib-includes=") {
+		$taglib_includes = substr($opt, 23);
   } elsif ($opt eq "--without-mp4v2") {
 		$have_mp4v2 = 0;
+  } elsif ($opt eq "--with-dbus") {
+		$have_qtdbus = 1;
 	} elsif ($opt eq "--without-musicbrainz") {
 		$have_tunepimp = 0;
 	} elsif (substr($opt, 0, 19) eq "--with-musicbrainz=") {
@@ -193,6 +210,20 @@ while (my $opt = shift) {
 		$docdir = substr($opt, 14);
 	} elsif (substr($opt, 0, 23) eq "--with-translationsdir=") {
 		$translationsdir = substr($opt, 23);
+	} elsif (substr($opt, 0, 22) eq "--with-extra-includes=") {
+		$extra_includes = substr($opt, 22);
+	} elsif (substr($opt, 0, 18) eq "--with-extra-libs=") {
+		$extra_libs = substr($opt, 18);
+	} elsif (substr($opt, 0, 21) eq "--with-extra-defines=") {
+		$extra_defines = substr($opt, 21);
+	} elsif (substr($opt, 0, 22) eq "--with-extra-cxxflags=") {
+		$extra_cxxflags = substr($opt, 22);
+	} elsif (substr($opt, 0, 15) eq "--with-db2html=") {
+		$db2html = substr($opt, 15);
+	} elsif (substr($opt, 0, 22) eq "--with-xsl-stylesheet=") {
+		$xsl_stylesheet = substr($opt, 22);
+	} elsif (substr($opt, 0, 16) eq "--with-perl-cmd=") {
+		$perl_cmd = substr($opt, 16);
 	} elsif ($opt eq "--from-configure") {
 		$from_configure = 1;
 	} elsif ($opt eq "--generate-ts") {
@@ -204,13 +235,23 @@ while (my $opt = shift) {
 		print "  --with-datarootdir=DIR data root [PREFIX/share]\n";
 		print "  --with-docdir=DIR      documentation [DATAROOTDIR/doc/kid3-qt]\n";
 		print "  --with-translationsdir=DIR translations [DATAROOTDIR/kid3-qt/translations]\n";
+		print "  --with-extra-includes=DIR adds non standard include paths\n";
+		print "  --with-extra-libs=LIB  adds non standard library options\n";
+		print "  --with-extra-defines=D adds non standard defines\n";
+		print "  --with-extra-cxxflags=F adds non standard compiler options\n";
+		print "  --with-db2html=PROGRAM Docbook to HTML command (e.g. xsltproc, jw, xalan)\n";
+		print "  --with-xsl-stylesheet=P path to docbook.xsl\n";
+		print "  --with-perl-cmd=PROGRAM perl command\n";
 		print "  --without-musicbrainz  build without MusicBrainz\n";
 		print "  --with-musicbrainz=VER build with MusicBrainz version [5]\n";
 		print "  --without-taglib       build without taglib\n";
+		print "  --with-taglib-includes=I taglib includes [-I/usr/include/taglib]\n";
 		print "  --without-mp4v2        build without mp4v2\n";
 		print "  --without-id3lib       build without id3lib\n";
 		print "  --without-vorbis       build without ogg/vorbis\n";
 		print "  --without-flac         build without FLAC\n";
+		print "  --without-flac-picture build without FLAC picture support\n";
+		print "  --with-dbus            build with QtDBus\n";
 		print "  --enable-gcc-pch       enable precompiled headers\n";
 		print "  --enable-debug         enables debug symbols\n";
 		print "  --with-qmake=PROGRAM   qmake command [qmake]\n";
@@ -249,6 +290,7 @@ if ($from_configure) {
 	$have_flac = 0;
 	$have_taglib = 0;
 	$have_mp4v2 = 0;
+	$have_qtdbus = 0;
 	$have_tunepimp = 0;
 	$qmake_cmd = "";
 	$lupdate_cmd = "";
@@ -279,6 +321,8 @@ if ($from_configure) {
 				$have_taglib = $1;
 			} elsif (/^#define HAVE_MP4V2 (\d+)$/) {
 				$have_mp4v2 = $1;
+			} elsif (/^#define HAVE_QTDBUS (\d+)$/) {
+				$have_qtdbus = $1;
 			} elsif (/^#define HAVE_TUNEPIMP (\d+)$/) {
 				$have_tunepimp = $1;
 			} elsif (/^#define CFG_QMAKE "(.*)"$/) {
@@ -295,7 +339,7 @@ if ($from_configure) {
 	}
 }
 
-my $config_h = "#define VERSION \"0.10\"\n";
+my $config_h = "#define VERSION \"1.0\"\n";
 my $config_pri;
 if ($prefix) {
 	$config_h .= "#define CFG_PREFIX \"$prefix\"\n";
@@ -317,6 +361,30 @@ if ($translationsdir) {
 	$config_h .= "#define CFG_TRANSLATIONSDIR \"$translationsdir\"\n";
 	$config_pri .= "CFG_TRANSLATIONSDIR = $translationsdir\n";
 }
+if ($taglib_includes) {
+	$config_pri .= "TAGLIB_INCLUDES = $taglib_includes\n";
+}
+if ($extra_includes) {
+	$config_pri .= "CFG_EXTRA_INCLUDES = $extra_includes\n";
+}
+if ($extra_libs) {
+	$config_pri .= "CFG_EXTRA_LIBS = $extra_libs\n";
+}
+if ($extra_defines) {
+	$config_pri .= "CFG_EXTRA_DEFINES = $extra_defines\n";
+}
+if ($extra_cxxflags) {
+	$config_pri .= "CFG_EXTRA_CXXFLAGS = $extra_cxxflags\n";
+}
+if ($db2html) {
+	$config_pri .= "CFG_DB2HTML = $db2html\n";
+}
+if ($xsl_stylesheet) {
+	$config_pri .= "CFG_XSL_STYLESHEET = $xsl_stylesheet\n";
+}
+if ($perl_cmd) {
+	$config_pri .= "CFG_PERL_CMD = $perl_cmd\n";
+}
 $config_pri .= "CFG_LIBS = ";
 
 my $allsys_h = <<"EOF";
@@ -350,6 +418,9 @@ if ($have_flac) {
 	$config_pri .= "-lFLAC++ -lFLAC ";
 	$allsys_h .= "#include <FLAC++/metadata.h>\n";
 }
+if ($have_flac_picture) {
+	$config_h .= "#define HAVE_FLAC_PICTURE $have_flac_picture\n";
+}
 if ($have_taglib) {
 	$config_h .= "#define HAVE_TAGLIB $have_taglib\n";
 	$config_pri .= "-ltag ";
@@ -357,6 +428,7 @@ if ($have_taglib) {
 if ($have_mp4v2) {
 	$config_h .= "#define HAVE_MP4V2 $have_mp4v2\n";
 	$config_pri .= "-lmp4v2 ";
+	$config_pri .= "-lwsock32 " if $^O eq "MSWin32" or $^O eq "msys";
 }
 if ($have_tunepimp) {
 	$config_h .= "#define HAVE_TUNEPIMP $have_tunepimp\n";
@@ -367,6 +439,10 @@ if ($have_tunepimp) {
 		$allsys_h .= "#include <tunepimp/tp_c.h>\n";
 	}
 }
+if ($have_qtdbus) {
+	$config_h .= "#define HAVE_QTDBUS $have_qtdbus\n";
+	$config_pri .= "\nHAVE_QTDBUS = $have_qtdbus";
+}
 
 $config_pri .= "\nCFG_CONFIG = " .
 	($enable_debug ? "debug" : "release") .
@@ -519,23 +595,27 @@ if ($generate_ts) {
 }
 
 print "starting $qmake_cmd\n";
-system "$qmake_cmd $topdir/kid3-qt.pro";
-chdir "kid3";
-system "$qmake_cmd $topdir/kid3/kid3.pro";
-chdir "..";
-chdir "po";
-system "$qmake_cmd $topdir/po/po.pro";
-chdir "..";
-chdir "doc";
-mkdir "en" unless -d "en";
-mkdir "de" unless -d "de";
-chdir "en";
-system "$qmake_cmd $topdir/doc/en/en.pro";
-chdir "..";
-chdir "de";
-system "$qmake_cmd $topdir/doc/de/de.pro";
-chdir "..";
-chdir "..";
+if ($^O eq "darwin") {
+	system "$qmake_cmd -spec macx-g++ $topdir/kid3-qt.pro";
+} else {
+	system "$qmake_cmd $topdir/kid3-qt.pro";
+	chdir "kid3";
+	system "$qmake_cmd $topdir/kid3/kid3.pro";
+	chdir "..";
+	chdir "po";
+	system "$qmake_cmd $topdir/po/po.pro";
+	chdir "..";
+	chdir "doc";
+	mkdir "en" unless -d "en";
+	mkdir "de" unless -d "de";
+	chdir "en";
+	system "$qmake_cmd $topdir/doc/en/en.pro";
+	chdir "..";
+	chdir "de";
+	system "$qmake_cmd $topdir/doc/de/de.pro";
+	chdir "..";
+	chdir "..";
+}
 
 if ($enable_pch) {
 # qmake < Qt 4.2 generates a dependency requiring the precompiled header
