@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 10 May 2006
  *
- * Copyright (C) 2006-2007  Urs Fleisch
+ * Copyright (C) 2006-2008  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -50,6 +50,7 @@
 #include <qapplication.h>
 #include <qclipboard.h>
 #include <qurl.h>
+#include <qtooltip.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
 #include <QGroupBox>
@@ -60,6 +61,25 @@
 #include <qgroupbox.h>
 #endif
 
+/**
+ * Get help text for format codes supported by formatString().
+ *
+ * @return help text.
+ */
+static QString getFormatToolTip()
+{
+	QString str;
+	str += "<table>\n";
+	str += ImportTrackData::getFormatToolTip(true);
+
+	str += "<tr><td>%n</td><td>%{tracks}</td><td>";
+	str += QCM_translate(I18N_NOOP("Number of tracks"));
+	str += "</td></tr>\n";
+
+	str += "</table>\n";
+	return str;
+}
+
 /**
  * Constructor.
  *
@@ -92,6 +112,10 @@ ExportDialog::ExportDialog(QWidget* parent) :
 			m_headerLineEdit = new QLineEdit(fmtbox);
 			m_trackLineEdit = new QLineEdit(fmtbox);
 			m_trailerLineEdit = new QLineEdit(fmtbox);
+			QString formatToolTip = getFormatToolTip();
+			QCM_setToolTip(m_headerLineEdit, formatToolTip);
+			QCM_setToolTip(m_trackLineEdit, formatToolTip);
+			QCM_setToolTip(m_trailerLineEdit, formatToolTip);
 #if QT_VERSION >= 0x040000
 			QVBoxLayout* vbox = new QVBoxLayout;
 			vbox->setMargin(2);
@@ -181,16 +205,13 @@ ExportDialog::~ExportDialog()
 
 /**
  * Export to a file.
+ *
+ * @param fn file name
+ *
+ * @return true if ok.
  */
-void ExportDialog::slotToFile()
+bool ExportDialog::exportToFile(const QString& fn)
 {
-	QString fn =
-#ifdef CONFIG_USE_KDE
-		KFileDialog::getSaveFileName(ImportSelector::getImportDir(),
-																 QString::null, this);
-#else
-	QFileDialog::QCM_getSaveFileName(this, ImportSelector::getImportDir());
-#endif
 	if (!fn.isEmpty()) {
 		QFile file(fn);
 		if (file.open(QCM_WriteOnly)) {
@@ -207,8 +228,25 @@ void ExportDialog::slotToFile()
 #endif
 			stream << m_edit->QCM_toPlainText();
 			file.close();
+			return true;
 		}
 	}
+	return false;
+}
+
+/**
+ * Export to a file.
+ */
+void ExportDialog::slotToFile()
+{
+	exportToFile(
+#ifdef CONFIG_USE_KDE
+		KFileDialog::getSaveFileName(ImportSelector::getImportDir(),
+																 QString::null, this)
+#else
+		QFileDialog::QCM_getSaveFileName(this, ImportSelector::getImportDir())
+#endif
+		);
 }
 
 /**
@@ -235,8 +273,12 @@ void ExportDialog::setFormatLineEdit(int index)
 		m_headerLineEdit->setText(m_formatHeaders[index]);
 		m_trackLineEdit->setText(m_formatTracks[index]);
 		m_trailerLineEdit->setText(m_formatTrailers[index]);
-		showPreview();
+	} else {
+		m_headerLineEdit->clear();
+		m_trackLineEdit->clear();
+		m_trailerLineEdit->clear();
 	}
+	showPreview();
 }
 
 /**
@@ -258,11 +300,12 @@ static QString trackDataToString(
 	if (!fmt.isEmpty()) {
 		const int numTagCodes = 1;
 		const QChar tagCode[numTagCodes] = { 'n' };
+		const QStringList tagLongCodes("tracks");
 		QString tagStr[numTagCodes];
 		tagStr[0] = QString::number(numTracks);
 
 		fmt = StandardTags::replacePercentCodes(
-			fmt, tagCode, tagStr, numTagCodes);
+			fmt, tagCode, tagLongCodes, tagStr, numTagCodes);
 	}
 	return fmt;
 }
@@ -309,12 +352,10 @@ void ExportDialog::setExportData(const ImportTrackDataVector& trackDataVector)
 }
 
 /**
- * Read the local settings from the configuration.
+ * Set the format combo box and line edits from the configuration.
  */
-void ExportDialog::readConfig()
+void ExportDialog::setFormatFromConfig()
 {
-	m_srcComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_exportSrcV1 ? SrcV1 : SrcV2);
-
 	m_formatHeaders = Kid3App::s_genCfg.m_exportFormatHeaders;
 	m_formatTracks = Kid3App::s_genCfg.m_exportFormatTracks;
 	m_formatTrailers = Kid3App::s_genCfg.m_exportFormatTrailers;
@@ -322,6 +363,16 @@ void ExportDialog::readConfig()
 	m_formatComboBox->QCM_addItems(Kid3App::s_genCfg.m_exportFormatNames);
 	m_formatComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_exportFormatIdx);
 	setFormatLineEdit(Kid3App::s_genCfg.m_exportFormatIdx);
+}
+
+/**
+ * Read the local settings from the configuration.
+ */
+void ExportDialog::readConfig()
+{
+	m_srcComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_exportSrcV1 ? SrcV1 : SrcV2);
+
+	setFormatFromConfig();
 
 	if (Kid3App::s_genCfg.m_exportWindowWidth > 0 &&
 			Kid3App::s_genCfg.m_exportWindowHeight > 0) {
@@ -337,13 +388,22 @@ void ExportDialog::saveConfig()
 {
 	Kid3App::s_genCfg.m_exportSrcV1 = (m_srcComboBox->QCM_currentIndex() == SrcV1);
 	Kid3App::s_genCfg.m_exportFormatIdx = m_formatComboBox->QCM_currentIndex();
-	Kid3App::s_genCfg.m_exportFormatNames[Kid3App::s_genCfg.m_exportFormatIdx] = m_formatComboBox->currentText();
-	Kid3App::s_genCfg.m_exportFormatHeaders[Kid3App::s_genCfg.m_exportFormatIdx] = m_headerLineEdit->text();
-	Kid3App::s_genCfg.m_exportFormatTracks[Kid3App::s_genCfg.m_exportFormatIdx] = m_trackLineEdit->text();
-	Kid3App::s_genCfg.m_exportFormatTrailers[Kid3App::s_genCfg.m_exportFormatIdx] = m_trailerLineEdit->text();
-
+	if (Kid3App::s_genCfg.m_exportFormatIdx < static_cast<int>(Kid3App::s_genCfg.m_exportFormatNames.size())) {
+		Kid3App::s_genCfg.m_exportFormatNames[Kid3App::s_genCfg.m_exportFormatIdx] = m_formatComboBox->currentText();
+		Kid3App::s_genCfg.m_exportFormatHeaders[Kid3App::s_genCfg.m_exportFormatIdx] = m_headerLineEdit->text();
+		Kid3App::s_genCfg.m_exportFormatTracks[Kid3App::s_genCfg.m_exportFormatIdx] = m_trackLineEdit->text();
+		Kid3App::s_genCfg.m_exportFormatTrailers[Kid3App::s_genCfg.m_exportFormatIdx] = m_trailerLineEdit->text();
+	} else {
+		Kid3App::s_genCfg.m_exportFormatIdx = Kid3App::s_genCfg.m_exportFormatNames.size();
+		Kid3App::s_genCfg.m_exportFormatNames.append(m_formatComboBox->currentText());
+		Kid3App::s_genCfg.m_exportFormatHeaders.append(m_headerLineEdit->text());
+		Kid3App::s_genCfg.m_exportFormatTracks.append(m_trackLineEdit->text());
+		Kid3App::s_genCfg.m_exportFormatTrailers.append(m_trailerLineEdit->text());
+	}
 	Kid3App::s_genCfg.m_exportWindowWidth = size().width();
 	Kid3App::s_genCfg.m_exportWindowHeight = size().height();
+
+	setFormatFromConfig();
 }
 
 /**
