@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 9 Jan 2003
  *
- * Copyright (C) 2003-2007  Urs Fleisch
+ * Copyright (C) 2003-2008  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -197,6 +197,16 @@ FileListItem* FileList::next()
 	return 0;
 }
 
+/**
+ * Get the current item in the filelist.
+ *
+ * @return current file.
+ */
+FileListItem* FileList::current()
+{
+	return dynamic_cast<FileListItem*>(currentItem());
+}
+
 /**
  * Get the next item in the current directory which contains a file.
  *
@@ -315,6 +325,79 @@ FileListItem* FileList::nextInDir()
 	return m_currentItemInDir;
 }
 
+/**
+ * Get the next item in the filelist which contains a file or a directory.
+ *
+ * @param it list view iterator
+ *
+ * @return next item with file or directory.
+ */
+static FileListItem* getNextItemWithFileOrDir(
+#if QT_VERSION >= 0x040000
+	QTreeWidgetItemIterator& it
+#else
+	QListViewItemIterator& it
+#endif
+	)
+{
+#if QT_VERSION >= 0x040000
+	QTreeWidgetItem* lvItem;
+#else
+	QListViewItem* lvItem;
+#endif
+	while ((lvItem = *it) != 0) {
+		FileListItem* flItem = dynamic_cast<FileListItem*>(lvItem);
+		if (flItem) {
+			return flItem;
+		}
+		++it;
+	}
+	return 0;
+}
+
+/**
+ * Get the first file or directory item in the filelist.
+ *
+ * @return first file.
+ */
+FileListItem* FileList::firstFileOrDir()
+{
+#if QT_VERSION >= 0x040000
+	delete m_iterator;
+	if (topLevelItemCount() > 0) {
+		m_iterator = new QTreeWidgetItemIterator(this);
+		return getNextItemWithFileOrDir(*m_iterator);
+	} else {
+		m_iterator = 0;
+		return 0;
+	}
+#else
+	m_iterator = QListViewItemIterator(this);
+	return getNextItemWithFileOrDir(m_iterator);
+#endif
+}
+
+/**
+ * Get the next file or directory item in the filelist.
+ *
+ * @return next file.
+ */
+FileListItem* FileList::nextFileOrDir()
+{
+#if QT_VERSION >= 0x040000
+	if (m_iterator && **m_iterator) {
+		++*m_iterator;
+		return getNextItemWithFileOrDir(*m_iterator);
+	}
+#else
+	if (*m_iterator) {
+		++m_iterator;
+		return getNextItemWithFileOrDir(m_iterator);
+	}
+#endif
+	return 0;
+}
+
 /**
  * Get the number of files selected in the filelist.
  *
@@ -340,9 +423,38 @@ int FileList::numFilesSelected()
 }
 
 /**
- * Select the next file.
+ * Select the first file.
+ *
+ * @return true if a file exists.
  */
-void FileList::selectNextFile()
+bool FileList::selectFirstFile()
+{
+#if QT_VERSION >= 0x040000
+	QTreeWidgetItem* item = *QTreeWidgetItemIterator(this);
+	if (item) {
+		clearSelection();
+		setCurrentItem(item);
+		setItemSelected(item, true);
+		return true;
+	}
+#else
+	QListViewItem* item = firstChild();
+	if (item) {
+		clearSelection();
+		setCurrentItem(item);
+		setSelected(item, true);
+		return true;
+	}
+#endif
+	return false;
+}
+
+/**
+ * Select the next file.
+ *
+ * @return true if a next file exists.
+ */
+bool FileList::selectNextFile()
 {
 #if QT_VERSION >= 0x040000
 	QTreeWidgetItem* item = currentItem();
@@ -350,6 +462,7 @@ void FileList::selectNextFile()
 		clearSelection();
 		setCurrentItem(item);
 		setItemSelected(item, true);
+		return true;
 	}
 #else
 	QListViewItem* item = currentItem();
@@ -357,14 +470,18 @@ void FileList::selectNextFile()
 		clearSelection();
 		setCurrentItem(item);
 		setSelected(item, true);
+		return true;
 	}
 #endif
+	return false;
 }
 
 /**
  * Select the previous file.
+ *
+ * @return true if a previous file exists.
  */
-void FileList::selectPreviousFile()
+bool FileList::selectPreviousFile()
 {
 #if QT_VERSION >= 0x040000
 	QTreeWidgetItem* item = currentItem();
@@ -372,6 +489,7 @@ void FileList::selectPreviousFile()
 		clearSelection();
 		setCurrentItem(item);
 		setItemSelected(item, true);
+		return true;
 	}
 #else
 	QListViewItem* item = currentItem();
@@ -379,8 +497,10 @@ void FileList::selectPreviousFile()
 		clearSelection();
 		setCurrentItem(item);
 		setSelected(item, true);
+		return true;
 	}
 #endif
+	return false;
 }
 
 /**
@@ -585,10 +705,10 @@ QStringList FileList::formatStringList(const QStringList& format)
 		if ((*it).QCM_indexOf('%') == -1) {
 			fmt.push_back(*it);
 		} else {
-			if (*it == "%F") {
+			if (*it == "%F" || *it == "%{files}") {
 				// list of files
 				fmt += files;
-			} else if (*it == "%uF") {
+			} else if (*it == "%uF" || *it == "%{urls}") {
 				// list of URLs or URL
 				QUrl url;
 				url.QCM_setScheme("file");
@@ -601,6 +721,8 @@ QStringList FileList::formatStringList(const QStringList& format)
 			} else {
 				const int numTagCodes = 3;
 				const QChar tagCode[numTagCodes] = { 'f', 'd', 'b' };
+				const QStringList tagLongCodes =
+					(QStringList() << "file" << "directory" << "browser");
 				QString tagStr[numTagCodes];
 				if (!files.empty()) {
 					tagStr[0] = files.front();
@@ -616,14 +738,24 @@ QStringList FileList::formatStringList(const QStringList& format)
 						}
 					}
 				}
-				QString str = StandardTags::replacePercentCodes(*it, tagCode, tagStr, numTagCodes);
+				QString str = StandardTags::replacePercentCodes(
+					*it, tagCode, tagLongCodes, tagStr, numTagCodes);
 
-				int ufPos;
-				if ((ufPos = str.QCM_indexOf("%uf")) != -1 && !files.empty()) {
+				int ufLen = 0;
+				int ufPos = str.QCM_indexOf("%uf");
+				if (ufPos != -1) {
+					ufLen = 3;
+				} else {
+					ufPos = str.QCM_indexOf("%{url}");
+					if (ufPos != -1) {
+						ufLen = 6;
+					}
+				}
+				if (ufPos != -1 && !files.empty()) {
 					QUrl url;
 					url.QCM_setScheme("file");
 					url.QCM_setPath(files.front());
-					str.replace(ufPos, 3, url.toString());
+					str.replace(ufPos, ufLen, url.toString());
 				}
 
 				if (firstSelectedItem) {
@@ -641,6 +773,52 @@ QStringList FileList::formatStringList(const QStringList& format)
 	return fmt;
 }
 
+/**
+ * Get help text for format codes supported by formatStringList().
+ *
+ * @param onlyRows if true only the tr elements are returned,
+ *                 not the surrounding table
+ *
+ * @return help text.
+ */
+QString FileList::getFormatToolTip(bool onlyRows)
+{
+	QString str;
+	if (!onlyRows) str += "<table>\n";
+	str += StandardTags::getFormatToolTip(true);
+
+	str += "<tr><td>%f</td><td>%{file}</td><td>";
+	str += QCM_translate("Filename");
+	str += "</td></tr>\n";
+
+	str += "<tr><td>%F</td><td>%{files}</td><td>";
+	str += QCM_translate(I18N_NOOP("Filenames"));
+	str += "</td></tr>\n";
+
+	str += "<tr><td>%uf</td><td>%{url}</td><td>";
+	str += QCM_translate("URL");
+	str += "</td></tr>\n";
+
+	str += "<tr><td>%uF</td><td>%{urls}</td><td>";
+	str += QCM_translate(I18N_NOOP("URLs"));
+	str += "</td></tr>\n";
+
+	str += "<tr><td>%d</td><td>%{directory}</td><td>";
+	str += QCM_translate(I18N_NOOP("Directory name"));
+	str += "</td></tr>\n";
+
+	str += "<tr><td>%b</td><td>%{browser}</td><td>";
+	str += QCM_translate("Browser");
+	str += "</td></tr>\n";
+
+	str += "<tr><td>%ua...</td><td>%u{artist}...</td><td>";
+	str += QCM_translate(I18N_NOOP("Encode as URL"));
+	str += "</td></tr>\n";
+
+	if (!onlyRows) str += "</table>\n";
+	return str;
+}
+
 /**
  * Execute a context menu command.
  *
