@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 02 Oct 2006
+ *
+ * Copyright (C) 2006-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "filelistitem.h"
@@ -117,39 +134,39 @@ static const char* const null_xpm[] = {
 /** picture with V1 and V2 */
 static const char* const v1v2_xpm[] = {
 	"16 16 3 1",
-	"       c None",
-	".      c #000000",
-	"+      c #FFFFFF",
+	"  c None",
+	". c #FFFFFF",
+	"+ c #000000",
 	"                ",
+	"  ..  ..   .    ",
+	"  .+  .+  .+    ",
+	"  .+  .+ .++    ",
+	"   .+.+ .+.+    ",
+	"   .+.+   .+    ",
+	"    .+    .+    ",
 	"                ",
-	"   .   .   .    ",
-	"   .   .  ..    ",
-	"    . .  . .    ",
-	"    . .    .    ",
-	"     .     .    ",
-	"                ",
-	"                ",
-	"   .   .  ..    ",
-	"   .   . .  .   ",
-	"    . .    .    ",
-	"    . .   .     ",
-	"     .   ....   ",
+	"  ..  ..  ..    ",
+	"  .+  .+ .++.   ",
+	"  .+  .+.+ .+   ",
+	"   .+.+   .+    ",
+	"   .+.+  .+..   ",
+	"    .+  .++++   ",
 	"                ",
 	"                "};
 
 /** picture with V1 */
 static const char* const v1_xpm[] = {
 	"16 16 3 1",
-	"       c None",
-	".      c #000000",
-	"+      c #FFFFFF",
+	"  c None",
+	". c #FFFFFF",
+	"+ c #000000",
 	"                ",
-	"                ",
-	"   .   .   .    ",
-	"   .   .  ..    ",
-	"    . .  . .    ",
-	"    . .    .    ",
-	"     .     .    ",
+	"  ..  ..   .    ",
+	"  .+  .+  .+    ",
+	"  .+  .+ .++    ",
+	"   .+.+ .+.+    ",
+	"   .+.+   .+    ",
+	"    .+    .+    ",
 	"                ",
 	"                ",
 	"                ",
@@ -163,9 +180,9 @@ static const char* const v1_xpm[] = {
 /** picture with V2 */
 static const char* const v2_xpm[] = {
 	"16 16 3 1",
-	"       c None",
-	".      c #000000",
-	"+      c #FFFFFF",
+	"  c None",
+	". c #FFFFFF",
+	"+ c #000000",
 	"                ",
 	"                ",
 	"                ",
@@ -174,51 +191,51 @@ static const char* const v2_xpm[] = {
 	"                ",
 	"                ",
 	"                ",
-	"                ",
-	"   .   .  ..    ",
-	"   .   . .  .   ",
-	"    . .    .    ",
-	"    . .   .     ",
-	"     .   ....   ",
+	"  ..  ..  ..    ",
+	"  .+  .+ .++.   ",
+	"  .+  .+.+ .+   ",
+	"   .+.+   .+    ",
+	"   .+.+  .+..   ",
+	"    .+  .++++   ",
 	"                ",
 	"                "};
 
 /** picture with NO TAG */
 static const char* const notag_xpm[] = {
 	"16 16 3 1",
-	"       c None",
-	".      c #000000",
-	"+      c #FFFFFF",
+	"  c None",
+	". c #FFFFFF",
+	"+ c #000000",
 	"                ",
+	"  ..  ..  ..    ",
+	"  .+. .+ .++.   ",
+	"  .++..+.+ .+   ",
+	"  .+.+.+.+ .+   ",
+	"  .+ .++.+..+   ",
+	"  .+  .+ .++    ",
 	"                ",
-	"   .   .  ..    ",
-	"   ..  . .  .   ",
-	"   . . . .  .   ",
-	"   .  .. .  .   ",
-	"   .   .  ..    ",
-	"                ",
-	"                ",
-	"  ...  .   ..   ",
-	"   .  . . .     ",
-	"   .  ... . ..  ",
-	"   .  . . .  .  ",
-	"   .  . .  ..   ",
+	" ....  .   ..   ",
+	" .+++ .+. .++   ",
+	"  .+ .+.+.+ ..  ",
+	"  .+ .+++.+.++  ",
+	"  .+ .+.+.+..+  ",
+	"  .+ .+.+ .++   ",
 	"                ",
 	"                "};
 
 /** picture with closed folder */
 static const char* folder_closed_xpm[]={
 	"16 16 10 1",
-	" 	c #0E279A",
-	".	c #2852AF",
-	"+	c #2B66D3",
-	"@	c #2A83FE",
-	"#	c #627CB1",
-	"$	c #5FA1FC",
-	"%	c #B0B2CA",
-	"&	c #89BEF9",
-	"*	c #CDE9FD",
-	"=	c #FCFFFC",
+	"  c #0E279A",
+	". c #2852AF",
+	"+ c #2B66D3",
+	"@ c #2A83FE",
+	"# c #627CB1",
+	"$ c #5FA1FC",
+	"% c #B0B2CA",
+	"& c #89BEF9",
+	"* c #CDE9FD",
+	"= c #FCFFFC",
 	"==++++++========",
 	"=+======+=======",
 	"+==*=*==#++++++=",
@@ -239,16 +256,16 @@ static const char* folder_closed_xpm[]={
 /** picture with open folder */
 static const char* folder_open_xpm[]={
 	"16 16 10 1",
-	" 	c #0E279A",
-	".	c #2852AF",
-	"+	c #2B66D3",
-	"@	c #2A83FE",
-	"#	c #627CB1",
-	"$	c #5FA1FC",
-	"%	c #B0B2CA",
-	"&	c #89BEF9",
-	"*	c #CDE9FD",
-	"=	c #FCFFFC",
+	"  c #0E279A",
+	". c #2852AF",
+	"+ c #2B66D3",
+	"@ c #2A83FE",
+	"# c #627CB1",
+	"$ c #5FA1FC",
+	"% c #B0B2CA",
+	"& c #89BEF9",
+	"* c #CDE9FD",
+	"= c #FCFFFC",
 	"==++++++========",
 	"=+======+=======",
 	"+==*=*==#++++++=",
@@ -275,7 +292,12 @@ static const char* folder_open_xpm[]={
  */
 FileListItem::FileListItem(FileList* parent, FileListItem* after,
 													 TaggedFile* file) :
-	Q3ListViewItem(parent, after), m_file(file), m_dirInfo(0)
+#if QT_VERSION >= 0x040000
+	QTreeWidgetItem(parent, after),
+#else
+	QListViewItem(parent, after),
+#endif
+	m_file(file), m_dirInfo(0)
 {
 	init();
 }
@@ -289,7 +311,12 @@ FileListItem::FileListItem(FileList* parent, FileListItem* after,
  */
 FileListItem::FileListItem(FileListItem* parent, FileListItem* after,
 													 TaggedFile* file) :
-	Q3ListViewItem(parent, after), m_file(file), m_dirInfo(0)
+#if QT_VERSION >= 0x040000
+	QTreeWidgetItem(parent, after), m_isOpen(false),
+#else
+	QListViewItem(parent, after),
+#endif
+	m_file(file), m_dirInfo(0)
 {
 	init();
 }
@@ -345,6 +372,27 @@ FileListItem::~FileListItem()
 	delete m_dirInfo;
 }
 
+#if QT_VERSION >= 0x040000
+/**
+ * Opens or closes an item.
+ *
+ * @param o true to open
+ */
+void FileListItem::setOpen(bool o)
+{
+	if (m_dirInfo) {
+		m_isOpen = o;
+		setIcon(0, o ? *folderOpenPixmap : *folderClosedPixmap);
+
+		if (o  && !childCount()) {
+			treeWidget()->setUpdatesEnabled(false);
+			FileList::readSubDirectory(m_dirInfo, this, 0);
+			treeWidget()->setUpdatesEnabled(true);
+		}
+		updateIcons();
+	}
+}
+#else
 /**
  * Paints the contents of one column of an item.
  *
@@ -360,14 +408,10 @@ void FileListItem::paintCell(QPainter* p, const QColorGroup& cg,
 	if (Kid3App::s_miscCfg.m_markTruncations &&
 			column == 0 && m_file && m_file->getTruncationFlags() != 0) {
 		QColorGroup g(cg);
-#if QT_VERSION >= 0x040000
-		g.setColor(QPalette::Window, Qt::red);
-#else
 		g.setColor(QColorGroup::Base, Qt::red);
-#endif
-		Q3ListViewItem::paintCell(p, g, column, width, align);
+		QListViewItem::paintCell(p, g, column, width, align);
 	} else {
-		Q3ListViewItem::paintCell(p, cg, column, width, align);
+		QListViewItem::paintCell(p, cg, column, width, align);
 	}
 }
 
@@ -401,6 +445,7 @@ void FileListItem::setup()
 	}
 	QListViewItem::setup();
 }
+#endif
 
 /**
  * Set tagged file.
@@ -449,7 +494,7 @@ void FileListItem::updateIcons()
 {
 	if (m_file) {
 		if (m_file->isChanged()) {
-			setPixmap(0, *modifiedPixmap);
+			QCM_setIcon(0, *modifiedPixmap);
 		} else {
 			static const QPixmap* tagpm[] = {
 				notagPixmap, v1Pixmap, v2Pixmap, v1v2Pixmap, nullPixmap
@@ -466,8 +511,23 @@ void FileListItem::updateIcons()
 					tagpmIdx |= 2;
 				}
 			}
-			setPixmap(0, *tagpm[tagpmIdx]);
+			QCM_setIcon(0, *tagpm[tagpmIdx]);
 		}
+#if QT_VERSION >= 0x040200
+		if (Kid3App::s_miscCfg.m_markTruncations &&
+				m_file->getTruncationFlags() != 0) {
+			setBackground(0, Qt::red);
+		} else {
+			setBackground(0, Qt::NoBrush);
+		}
+#elif QT_VERSION >= 0x040000
+		if (Kid3App::s_miscCfg.m_markTruncations &&
+				m_file->getTruncationFlags() != 0) {
+			setBackgroundColor(0, Qt::red);
+		} else {
+			setBackgroundColor(0, QColor());
+		}
+#endif
 	}
 }
 
@@ -496,3 +556,16 @@ void FileListItem::setDirName(const QString& dirName)
 		setOpen(false);
 	}
 }
+
+#if QT_VERSION >= 0x040000
+/**
+ * Check if QTreeWidgetItem is selected.
+ *
+ * @return true if selected.
+ */
+bool FileListItem::isSelected() const
+{
+	const QTreeWidget* tree = treeWidget();
+	return tree && tree->isItemSelected(this);
+}
+#endif
