@@ -5,6 +5,23 @@
  * \b Project: Kid3
  * \author Urs Fleisch
  * \date 02 Oct 2006
+ *
+ * Copyright (C) 2006-2007  Urs Fleisch
+ *
+ * This file is part of Kid3.
+ *
+ * Kid3 is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Kid3 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef FILELISTITEM_H
@@ -13,7 +30,7 @@
 #include <qpixmap.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
-#include <Q3ListView>
+#include <QTreeWidget>
 #else
 #include <qlistview.h>
 #endif
@@ -24,7 +41,13 @@ class DirInfo;
 class FileList;
 
 /** List box item containing tagged file */
-class FileListItem : public Q3ListViewItem {
+class FileListItem : public
+#if QT_VERSION >= 0x040000
+QTreeWidgetItem
+#else
+QListViewItem
+#endif
+{
 public:
 	/**
 	 * Constructor.
@@ -49,6 +72,7 @@ public:
 	 */
 	virtual ~FileListItem();
 
+#if QT_VERSION < 0x040000
 	/**
 	 * Paints the contents of one column of an item.
 	 *
@@ -61,6 +85,12 @@ public:
 	virtual void paintCell(QPainter* p, const QColorGroup& cg,
 												 int column, int width, int align);
 
+	/**
+	 * Called before showing the item.
+	 */
+	virtual void setup();
+#endif
+
 	/**
 	 * Opens or closes an item.
 	 *
@@ -68,11 +98,6 @@ public:
 	 */
 	virtual void setOpen(bool o);
 
-	/**
-	 * Called before showing the item.
-	 */
-	void setup();
-
 	/**
 	 * Get tagged file.
 	 * @return tagged file.
@@ -138,6 +163,21 @@ public:
 	 */
 	bool isInSelection() { return m_selected; }
 
+#if QT_VERSION >= 0x040000
+	/**
+	 * Check if QTreeWidgetItem is selected.
+	 *
+	 * @return true if selected.
+	 */
+	bool isSelected() const;
+
+	/**
+	 * Check if directory is open.
+	 * @return true if open.
+	 */
+	bool isOpen() const { return m_isOpen; }
+#endif
+
 private:
 	FileListItem(const FileListItem&);
 	FileListItem& operator=(const FileListItem&);
@@ -148,6 +188,11 @@ private:
 	 */
 	void init();
 
+#if QT_VERSION >= 0x040000
+	/** true if directory open. */
+	bool m_isOpen;
+#endif
+
 	/** the tagged file represented by this item */
 	TaggedFile* m_file;
 
