@@ -87,6 +87,7 @@ const char* Frame::getNameFromType(Type type)
 		I18N_NOOP("Original Date"),   // FT_OriginalDate,
 		I18N_NOOP("Part"),            // FT_Part,
 		I18N_NOOP("Performer"),       // FT_Performer,
+		I18N_NOOP("Picture"),         // FT_Picture,
 		I18N_NOOP("Publisher"),       // FT_Publisher,
 		I18N_NOOP("Subtitle"),        // FT_Subtitle,
 		I18N_NOOP("Website")          // FT_Website,
@@ -98,6 +99,32 @@ const char* Frame::getNameFromType(Type type)
 	return type <= FT_LastFrame ? names[type] : "Unknown";
 }
 
+/**
+ * Get type of frame from English name.
+ *
+ * @param name name, spaces and case are ignored
+ *
+ * @return type.
+ */
+Frame::Type Frame::getTypeFromName(QString name)
+{
+	static QMap<QString, int> strNumMap;
+	if (strNumMap.empty()) {
+		// first time initialization
+		for (int i = 0; i <= FT_LastFrame; ++i) {
+			Type type = static_cast<Type>(i);
+			strNumMap.insert(QString(getNameFromType(type)).
+											 remove(' ').QCM_toUpper(), type);
+		}
+	}
+	QMap<QString, int>::const_iterator it =
+		strNumMap.find(name.remove(' ').QCM_toUpper());
+	if (it != strNumMap.end()) {
+		return static_cast<Type>(*it);
+	}
+	return FT_Other;
+}
+
 /**
  * Get name of frame.
  *
@@ -237,6 +264,31 @@ void FrameCollection::removeDisabledFrames(const FrameFilter& flt)
 	}
 }
 
+/**
+ * Find a frame by name.
+ *
+ * @param name  the name of the frame to find, if the exact name is not
+ *              found, a case-insensitive search for the first name
+ *              starting with this string is performed
+ *
+ * @return iterator or end() if not found.
+ */
+FrameCollection::iterator FrameCollection::findByName(const QString& name) const
+{
+	Frame::Type type = Frame::getTypeFromName(name);
+	Frame frame(type, "", name, -1);
+	const_iterator it = find(frame);
+	if (it == end()) {
+		QString ucName = name.QCM_toUpper();
+		int len = ucName.length();
+		for (it = begin(); it != end(); ++it) {
+			if (ucName == it->getName().QCM_toUpper().left(len)) {
+				break;
+			}
+		}
+	}
+	return it;
+}
 
 /**
  * Constructor.
