@@ -34,6 +34,9 @@
 #include <qpushbutton.h>
 #include <qfile.h>
 #include <qinputdialog.h>
+#include <qapplication.h>
+#include <qclipboard.h>
+#include <qbuffer.h>
 #if QT_VERSION >= 0x040000
 #include <QListWidget>
 #include <QVBoxLayout>
@@ -701,22 +704,69 @@ BinaryOpenSave::BinaryOpenSave(QWidget* parent, const Frame::Field& field) :
 {
 	QHBoxLayout* layout = new QHBoxLayout(this);
 	m_label = new QLabel(this);
+	m_clipButton = new QPushButton(i18n("From Clip&board"), this);
 	QPushButton* openButton = new QPushButton(i18n("&Import"), this);
 	QPushButton* saveButton = new QPushButton(i18n("&Export"), this);
 	QPushButton* viewButton = new QPushButton(i18n("&View"), this);
-	if (layout && m_label && openButton && saveButton && viewButton) {
+	if (layout && m_label && m_clipButton && openButton && saveButton && viewButton) {
 		layout->setMargin(0);
 		layout->setSpacing(6);
 		layout->addWidget(m_label);
+		layout->addWidget(m_clipButton);
 		layout->addWidget(openButton);
 		layout->addWidget(saveButton);
 		layout->addWidget(viewButton);
+		connect(m_clipButton, SIGNAL(clicked()), this, SLOT(clipData()));
 		connect(openButton, SIGNAL(clicked()), this, SLOT(loadData()));
 		connect(saveButton, SIGNAL(clicked()), this, SLOT(saveData()));
 		connect(viewButton, SIGNAL(clicked()), this, SLOT(viewData()));
+		connect(QApplication::clipboard(), SIGNAL(dataChanged()), this, SLOT(setClipButtonState()));
+		setClipButtonState();
 	}
 }
 
+/**
+ * Enable the "From Clipboard" button if the clipboard contains an image.
+ */
+void BinaryOpenSave::setClipButtonState()
+{
+	QClipboard* cb = QApplication::clipboard();
+	m_clipButton->setEnabled(
+#if QT_VERSION >= 0x040000
+		cb && (cb->mimeData()->hasFormat("image/jpeg") ||
+					 cb->mimeData()->hasImage())
+#else
+		cb && cb->data(QClipboard::Clipboard)->provides("image/jpeg")
+#endif
+		);
+}
+
+/**
+ * Load image from clipboard.
+ */
+void BinaryOpenSave::clipData()
+{
+	QClipboard* cb = QApplication::clipboard();
+#if QT_VERSION >= 0x040000
+	if (cb) {
+		if (cb->mimeData()->hasFormat("image/jpeg")) {
+			m_byteArray = cb->mimeData()->data("image/jpeg");
+			m_isChanged = true;
+		} else if (cb->mimeData()->hasImage()) {
+			QBuffer buffer(&m_byteArray);
+			buffer.open(QIODevice::WriteOnly);
+			cb->image().save(&buffer, "JPG");
+			m_isChanged = true;
+		}
+	}
+#else
+	if (cb && cb->data(QClipboard::Clipboard)->provides("image/jpeg")) {
+		m_byteArray = cb->data(QClipboard::Clipboard)->encodedData("image/jpeg");
+		m_isChanged = true;
+	}
+#endif
+}
+
 /**
  * Request name of file to import binary data from.
  * The data is imported later when Ok is pressed in the parent dialog.
@@ -1344,6 +1394,12 @@ bool FrameList::editFrame(Frame& frame)
 	bool result = true;
 	QString name(frame.getName(true));
 	if (!name.isEmpty()) {
+		int nlPos = name.QCM_indexOf("\n");
+		if (nlPos > 0) {
+			// probably "TXXX - User defined text information\nDescription" or
+			// "WXXX - User defined URL link\nDescription"
+			name.truncate(nlPos);
+		}
 #if QT_VERSION >= 0x040000
 		name = QCM_translate(name.toLatin1().data());
 #else
@@ -1440,13 +1496,13 @@ bool FrameList::addFrame(bool edit)
 }
 
 /**
- * Get type of frame from name.
+ * Get type of frame from translated name.
  *
  * @param name name, spaces and case are ignored
  *
  * @return type.
  */
-static Frame::Type getTypeFromName(QString name)
+static Frame::Type getTypeFromTranslatedName(QString name)
 {
 	static QMap<QString, int> strNumMap;
 	if (strNumMap.empty()) {
@@ -1481,7 +1537,7 @@ bool FrameList::selectFrame()
 			0, QCM_translate(title),
 			QCM_translate(msg), m_file->getFrameIds(), 0, true, &ok);
 		if (ok) {
-			Frame::Type type = getTypeFromName(name);
+			Frame::Type type = getTypeFromTranslatedName(name);
 			m_frame = Frame(type, "", name, -1);
 		}
 	}
