@@ -95,6 +95,7 @@ void FrameTableLineEdit::focusInEvent(QFocusEvent* event)
 	QLineEdit::focusInEvent(event);
 }
 
+/** Combo box which sets current table cell in EditType Always. */
 class FrameTableComboBox : public QComboBox {
 public:
 	/**
@@ -604,6 +605,11 @@ void GenreTableItem::setText(const QString& t)
 	QWidget* w = table()->cellWidget(row(), col());
 	QComboBox* cb = ::qt_cast<QComboBox*>(w);
 	if (cb) {
+		int genreIndex = t.isNull() ? 0 :
+			Genres::getIndex(Genres::getNumber(t));
+		cb->setCurrentItem(
+			Kid3App::s_miscCfg.m_onlyCustomGenres ? 0 :
+			(genreIndex > 0 ? genreIndex : Genres::count + 1));
 		cb->setCurrentText(t);
 	}
 }
@@ -717,7 +723,12 @@ FrameTable::~FrameTable() {}
 QString FrameTable::getDisplayName(const QString& str) const
 {
 	if (!str.isEmpty()) {
-		if (str.mid(4, 3) == " - ") {
+		int nlPos = str.QCM_indexOf("\n");
+		if (nlPos > 0) {
+			// probably "TXXX - User defined text information\nDescription" or
+			// "WXXX - User defined URL link\nDescription"
+			return str.mid(nlPos + 1);
+		} else if (str.mid(4, 3) == " - ") {
 			// probably "ID3-ID - Description"
 			return str.left(4);
 		} else {
@@ -873,8 +884,11 @@ void FrameTable::framesToTable()
 
 /**
  * Set frames from values in table.
+ *
+ * @param setUnchanged if true, also set marked values which are unchanged,
+ *                     which can be used if multiple files are selected
  */
-void FrameTable::tableToFrames()
+void FrameTable::tableToFrames(bool setUnchanged)
 {
 #if QT_VERSION >= 0x040000
 	int row = 0;
@@ -886,8 +900,9 @@ void FrameTable::tableToFrames()
 		if ((twi = item(row, CI_Enable)) != 0 &&
 				twi->checkState() == Qt::Checked &&
 				(twi = item(row, CI_Value)) != 0 &&
-				(value = twi->text()) != (*it).getValue()) {
+				((value = twi->text()) != (*it).getValue() || setUnchanged)) {
 			Frame& frame = const_cast<Frame&>(*it);
+			if (value.isNull()) value = "";
 			frame.setValue(value);
 			frame.setValueChanged();
 		}
@@ -912,8 +927,9 @@ void FrameTable::tableToFrames()
 			} else {
 				value = text(row, CI_Value);
 			}
-			if (value != (*it).getValue() &&
-					!(value.isEmpty() && (*it).getValue().isEmpty())) {
+			if (setUnchanged ||
+					(value != (*it).getValue() &&
+					 !(value.isEmpty() && (*it).getValue().isEmpty()))) {
 				Frame& frame = const_cast<Frame&>(*it);
 				frame.setValue(value);
 				frame.setValueChanged();
@@ -1008,7 +1024,7 @@ bool FrameTable::selectFrameWithIndex(int index)
 /**
  * Select the row of the frame with a given name.
  *
- * @param index name of frame
+ * @param name name of frame
  *
  * @return true if found.
  */
@@ -1106,7 +1122,8 @@ bool FrameTable::eventFilter(QObject* o, QEvent* e)
 {
 	if (e && e->type() == QEvent::KeyPress) {
 		QKeyEvent* ke = (QKeyEvent*)e;
-		if (ke->key() == Key_Left || ke->key() == Key_Right) {
+		if (ke->key() == Key_Left || ke->key() == Key_Right ||
+				ke->key() == Key_Home || ke->key() == Key_End) {
 			return false;
 		} else if (ke->key() == Key_Tab || ke->key() == Key_BackTab) {
 			int row = currentRow(), col = currentColumn();
