@@ -43,48 +43,55 @@ ImportConfig::ImportConfig(const QString& grp) :
 	GeneralConfig(grp), m_importServer(ImportConfig::ServerFreedb),
 	m_importDest(ImportConfig::DestV1), m_importFormatIdx(0),
 	m_enableTimeDifferenceCheck(true), m_maxTimeDifference(3),
+	m_importWindowWidth(-1), m_importWindowHeight(-1),
 	m_exportSrcV1(true), m_exportFormatIdx(0),
 	m_exportWindowWidth(-1), m_exportWindowHeight(-1)
 {
 	/**
 	 * Preset import format regular expressions.
 	 * The following codes are used before the () expressions.
-	 * %s title (song)
-	 * %l album
-	 * %a artist
-	 * %c comment
-	 * %y year
-	 * %t track
-	 * %g genre
+	 * %s %{title} title (song)
+	 * %l %{album} album
+	 * %a %{artist} artist
+	 * %c %{comment} comment
+	 * %y %{year} year
+	 * %t %{track} track, at least two digits
+	 * %T %{tracknumber} track number
+	 * %g %{genre} genre
+	 * %d %{duration} duration mm:ss
+	 * %D %{seconds} duration in seconds
+	 * %f %{file} file name
+	 * %p %{filepath} absolute file path
+	 * %u %{url} URL
+	 * %n %{tracks} number of tracks
 	 */
 	m_importFormatNames.append("CSV unquoted");
 	m_importFormatHeaders.append("");
-	m_importFormatTracks.append("%t(\\d+)\\t%s([^\\r\\n\\t]*)\\t%a([^\\r\\n\\t]*)\\t%l([^\\r\\n\\t]*)\\t%y(\\d+)\\t%g([^\\r\\n\\t]*)\\t%c([^\\r\\n\\t]*)\\t(?:\\d+:)?%d(\\d+:\\d+)");
+	m_importFormatTracks.append("%{track}(\\d+)\\t%{title}([^\\r\\n\\t]*)\\t%{artist}([^\\r\\n\\t]*)\\t%{album}([^\\r\\n\\t]*)\\t%{year}(\\d+)\\t%{genre}([^\\r\\n\\t]*)\\t%{comment}([^\\r\\n\\t]*)\\t(?:\\d+:)?%{duration}(\\d+:\\d+)");
 
 	m_importFormatNames.append("CSV quoted");
 	m_importFormatHeaders.append("");
-	m_importFormatTracks.append("\"?%t(\\d+)\"?\\t\"?%s([^\\r\\n\\t\"]*)\"?\\t\"?%a([^\\r\\n\\t\"]*)\"?\\t\"?%l([^\\r\\n\\t\"]*)\"?\\t\"?%y(\\d+)\"?\\t\"?%g([^\\r\\n\\t\"]*)\"?\\t\"?%c([^\\r\\n\\t\"]*)\"?\\t\"?(?:\\d+:)?%d(\\d+:\\d+)");
-//	m_importFormatTracks.append("\"%t(\\d+)\"\\t\"%s([^\\r\\n]*)\"\\t\"%a([^\\r\\n]*)\"\\t\"%l([^\\r\\n]*)\"\\t\"%y(\\d+)\"\\t\"%g([^\\r\\n]*)\"\\t\"%c([^\\r\\n]*)\"\\t\"%d(\\d+:\\d+)\\.00\"");
+	m_importFormatTracks.append("\"?%{track}(\\d+)\"?\\t\"?%{title}([^\\r\\n\\t\"]*)\"?\\t\"?%{artist}([^\\r\\n\\t\"]*)\"?\\t\"?%{album}([^\\r\\n\\t\"]*)\"?\\t\"?%{year}(\\d+)\"?\\t\"?%{genre}([^\\r\\n\\t\"]*)\"?\\t\"?%{comment}([^\\r\\n\\t\"]*)\"?\\t\"?(?:\\d+:)?%{duration}(\\d+:\\d+)");
 
 	m_importFormatNames.append("freedb HTML text");
-	m_importFormatHeaders.append("%a(\\S[^\\r\\n/]*\\S)\\s*/\\s*%l(\\S[^\\r\\n]*\\S)[\\r\\n]+\\s*tracks:\\s+\\d+.*year:\\s*%y(\\d+)?.*genre:\\s*%g(\\S[^\\r\\n]*\\S)?[\\r\\n]");
-	m_importFormatTracks.append("[\\r\\n]%t(\\d+)[\\.\\s]+%d(\\d+:\\d+)\\s+%s(\\S[^\\r\\n]*\\S)");
+	m_importFormatHeaders.append("%{artist}(\\S[^\\r\\n/]*\\S)\\s*/\\s*%{album}(\\S[^\\r\\n]*\\S)[\\r\\n]+\\s*tracks:\\s+\\d+.*year:\\s*%{year}(\\d+)?.*genre:\\s*%{genre}(\\S[^\\r\\n]*\\S)?[\\r\\n]");
+	m_importFormatTracks.append("[\\r\\n]%{track}(\\d+)[\\.\\s]+%{duration}(\\d+:\\d+)\\s+%{title}(\\S[^\\r\\n]*\\S)");
 
 	m_importFormatNames.append("freedb HTML source");
-	m_importFormatHeaders.append("<[^>]+>%a([^<\\s][^\\r\\n/]*\\S)\\s*/\\s*%l(\\S[^\\r\\n]*[^\\s>])<[^>]+>[\\r\\n]+\\s*tracks:\\s+\\d+.*year:\\s*%y(\\d+)?.*genre:\\s*%g(\\S[^\\r\\n>]*\\S)?<[^>]+>[\\r\\n]");
-	m_importFormatTracks.append("<td[^>]*>\\s*%t(\\d+).</td><td[^>]*>\\s*%d(\\d+:\\d+)</td><td[^>]*>(?:<[^>]+>)?%s([^<\\r\\n]+)");
+	m_importFormatHeaders.append("<[^>]+>%{artist}([^<\\s][^\\r\\n/]*\\S)\\s*/\\s*%{album}(\\S[^\\r\\n]*[^\\s>])<[^>]+>[\\r\\n]+\\s*tracks:\\s+\\d+.*year:\\s*%{year}(\\d+)?.*genre:\\s*%{genre}(\\S[^\\r\\n>]*\\S)?<[^>]+>[\\r\\n]");
+	m_importFormatTracks.append("<td[^>]*>\\s*%{track}(\\d+).</td><td[^>]*>\\s*%{duration}(\\d+:\\d+)</td><td[^>]*>(?:<[^>]+>)?%{title}([^<\\r\\n]+)");
 
 	m_importFormatNames.append("Title");
 	m_importFormatHeaders.append("");
-	m_importFormatTracks.append("\\s*%s(\\S[^\\r\\n]*\\S)\\s*");
+	m_importFormatTracks.append("\\s*%{title}(\\S[^\\r\\n]*\\S)\\s*");
 
 	m_importFormatNames.append("Track Title");
 	m_importFormatHeaders.append("");
-	m_importFormatTracks.append("\\s*%t(\\d+)[\\.\\s]+%s(\\S[^\\r\\n]*\\S)\\s*");
+	m_importFormatTracks.append("\\s*%{track}(\\d+)[\\.\\s]+%{title}(\\S[^\\r\\n]*\\S)\\s*");
 
 	m_importFormatNames.append("Track Title Time");
 	m_importFormatHeaders.append("");
-	m_importFormatTracks.append("\\s*%t(\\d+)[\\.\\s]+%s(\\S[^\\r\\n]*\\S)\\s+%d(\\d+:\\d+)\\s*");
+	m_importFormatTracks.append("\\s*%{track}(\\d+)[\\.\\s]+%{title}(\\S[^\\r\\n]*\\S)\\s+%{duration}(\\d+:\\d+)\\s*");
 
 	m_importFormatNames.append("Custom Format");
 	m_importFormatHeaders.append("");
@@ -92,32 +99,32 @@ ImportConfig::ImportConfig(const QString& grp) :
 
 	m_exportFormatNames.append("CSV unquoted");
 	m_exportFormatHeaders.append("");
-	m_exportFormatTracks.append("%t\\t%s\\t%a\\t%l\\t%y\\t%g\\t%c\\t%d.00");
+	m_exportFormatTracks.append("%{track}\\t%{title}\\t%{artist}\\t%{album}\\t%{year}\\t%{genre}\\t%{comment}\\t%{duration}.00");
 	m_exportFormatTrailers.append("");
 
 	m_exportFormatNames.append("CSV quoted");
 	m_exportFormatHeaders.append("");
-	m_exportFormatTracks.append("\"%t\"\\t\"%s\"\\t\"%a\"\\t\"%l\"\\t\"%y\"\\t\"%g\"\\t\"%c\"\\t\"%d.00\"");
+	m_exportFormatTracks.append("\"%{track}\"\\t\"%{title}\"\\t\"%{artist}\"\\t\"%{album}\"\\t\"%{year}\"\\t\"%{genre}\"\\t\"%{comment}\"\\t\"%{duration}.00\"");
 	m_exportFormatTrailers.append("");
 
 	m_exportFormatNames.append("Extended M3U");
 	m_exportFormatHeaders.append("#EXTM3U");
-	m_exportFormatTracks.append("#EXTINF:%D,%a - %s\\n%p");
+	m_exportFormatTracks.append("#EXTINF:%{seconds},%{artist} - %{title}\\n%{filepath}");
 	m_exportFormatTrailers.append("");
 
 	m_exportFormatNames.append("Extended PLS");
 	m_exportFormatHeaders.append("[playlist]");
-	m_exportFormatTracks.append("File%T=%p\\nTitle%T=%a - %s\\nLength%T=%D");
-	m_exportFormatTrailers.append("NumberOfEntries=%n\\nVersion=2");
+	m_exportFormatTracks.append("File%{tracknumber}=%{filepath}\\nTitle%{tracknumber}=%{artist} - %{title}\\nLength%{tracknumber}=%{seconds}");
+	m_exportFormatTrailers.append("NumberOfEntries=%{tracks}\\nVersion=2");
 
 	m_exportFormatNames.append("HTML");
-	m_exportFormatHeaders.append("<html>\\n <head>\\n  <title>%a - %l</title>\\n </head>\\n <body>\\n  <h1>%a - %l</h1>\\n  <dl>");
-	m_exportFormatTracks.append("   <dt><a href=\"%u\">%t. %s</a></dt>");
+	m_exportFormatHeaders.append("<html>\\n <head>\\n  <title>%{artist} - %{album}</title>\\n </head>\\n <body>\\n  <h1>%{artist} - %{album}</h1>\\n  <dl>");
+	m_exportFormatTracks.append("   <dt><a href=\"%{url}\">%{track}. %{title}</a></dt>");
 	m_exportFormatTrailers.append("  </dl>\\n </body>\\n</html>");
 
 	m_exportFormatNames.append("Kover XML");
-	m_exportFormatHeaders.append("<kover>\\n <title>\\n  <text><![CDATA[%a ]]></text>\\n  <text><![CDATA[%l]]></text>\\n </title>\\n <content>");
-	m_exportFormatTracks.append("  <text><![CDATA[%t. %s]]></text>");
+	m_exportFormatHeaders.append("<kover>\\n <title>\\n  <text><![CDATA[%{artist} ]]></text>\\n  <text><![CDATA[%{album}]]></text>\\n </title>\\n <content>");
+	m_exportFormatTracks.append("  <text><![CDATA[%{track}. %{title}]]></text>");
 	m_exportFormatTrailers.append(" </content>\\n</kover>");
 
 	m_exportFormatNames.append("Custom Format");
@@ -154,6 +161,8 @@ void ImportConfig::writeToConfig(
 	cfg.writeEntry("ImportFormatIdx", m_importFormatIdx);
 	cfg.writeEntry("EnableTimeDifferenceCheck", m_enableTimeDifferenceCheck);
 	cfg.writeEntry("MaxTimeDifference", m_maxTimeDifference);
+	cfg.writeEntry("ImportWindowWidth", m_importWindowWidth);
+	cfg.writeEntry("ImportWindowHeight", m_importWindowHeight);
 
 	cfg.writeEntry("ExportSourceV1", m_exportSrcV1);
 	cfg.writeEntry("ExportFormatNames", m_exportFormatNames);
@@ -173,6 +182,8 @@ void ImportConfig::writeToConfig(
 	config->QCM_writeEntry("/ImportFormatIdx", m_importFormatIdx);
 	config->QCM_writeEntry("/EnableTimeDifferenceCheck", m_enableTimeDifferenceCheck);
 	config->QCM_writeEntry("/MaxTimeDifference", m_maxTimeDifference);
+	config->QCM_writeEntry("/ImportWindowWidth", m_importWindowWidth);
+	config->QCM_writeEntry("/ImportWindowHeight", m_importWindowHeight);
 
 	config->QCM_writeEntry("/ExportSourceV1", m_exportSrcV1);
 	config->QCM_writeEntry("/ExportFormatNames", m_exportFormatNames);
@@ -214,6 +225,8 @@ void ImportConfig::readFromConfig(
 	m_importFormatIdx = cfg.KCM_readNumEntry("ImportFormatIdx", m_importFormatIdx);
 	m_enableTimeDifferenceCheck = cfg.KCM_readBoolEntry("EnableTimeDifferenceCheck", m_enableTimeDifferenceCheck);
 	m_maxTimeDifference = cfg.KCM_readNumEntry("MaxTimeDifference", m_maxTimeDifference);
+	m_importWindowWidth = cfg.KCM_readNumEntry("ImportWindowWidth", -1);
+	m_importWindowHeight = cfg.KCM_readNumEntry("ImportWindowHeight", -1);
 
 	m_exportSrcV1 = cfg.KCM_readBoolEntry("ExportSourceV1", m_exportSrcV1);
 	expNames = cfg.KCM_readListEntry("ExportFormatNames");
@@ -245,6 +258,8 @@ void ImportConfig::readFromConfig(
 	m_importFormatIdx = config->QCM_readNumEntry("/ImportFormatIdx", m_importFormatIdx);
 	m_enableTimeDifferenceCheck = config->QCM_readBoolEntry("/EnableTimeDifferenceCheck", m_enableTimeDifferenceCheck);
 	m_maxTimeDifference = config->QCM_readNumEntry("/MaxTimeDifference", m_maxTimeDifference);
+	m_importWindowWidth = config->QCM_readNumEntry("/ImportWindowWidth", -1);
+	m_importWindowHeight = config->QCM_readNumEntry("/ImportWindowHeight", -1);
 
 	m_exportSrcV1 = config->QCM_readBoolEntry("/ExportSourceV1", m_exportSrcV1);
 	expNames = config->QCM_readListEntry("/ExportFormatNames");
@@ -272,7 +287,7 @@ void ImportConfig::readFromConfig(
 		if (idx >= 0) {
 			m_importFormatHeaders[idx] = *headersIt;
 			m_importFormatTracks[idx] = *tracksIt;
-		} else {
+		} else if (!(*namesIt).isEmpty()) {
 			m_importFormatNames.append(*namesIt);
 			m_importFormatHeaders.append(*headersIt);
 			m_importFormatTracks.append(*tracksIt);
@@ -295,7 +310,7 @@ void ImportConfig::readFromConfig(
 			m_exportFormatHeaders[idx] = *expHeadersIt;
 			m_exportFormatTracks[idx] = *expTracksIt;
 			m_exportFormatTrailers[idx] = *expTrailersIt;
-		} else {
+		} else if (!(*expNamesIt).isEmpty()) {
 			m_exportFormatNames.append(*expNamesIt);
 			m_exportFormatHeaders.append(*expHeadersIt);
 			m_exportFormatTracks.append(*expTracksIt);
