@@ -30,6 +30,7 @@
 #include <qstring.h>
 #include <qregexp.h>
 #include "qtcompatmac.h"
+/** The list type depends on the Qt version. */
 #if QT_VERSION >= 0x040000
 #include <QList>
 typedef QList<int> TrackDurationList;
@@ -71,6 +72,13 @@ public:
 	 */
 	TrackDurationList* getTrackDurations() { return &m_trackDuration; }
 
+	/**
+	 * Get help text for format codes supported by setFormat().
+	 *
+	 * @return help text.
+	 */
+	static QString getFormatToolTip();
+
 private:
 	/** track regexp pattern */
 	QString m_pattern;
