@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 17 Sep 2003
  *
- * Copyright (C) 2003-2007  Urs Fleisch
+ * Copyright (C) 2003-2008  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -42,6 +42,7 @@
 #include <qbitarray.h>
 #include <qcheckbox.h>
 #include <qspinbox.h>
+#include <qtooltip.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
 #include <QTableWidget>
@@ -257,6 +258,9 @@ ImportSelector::ImportSelector(
 	m_formatComboBox->setEditable(true);
 	m_headerLineEdit = new QLineEdit(fmtbox);
 	m_trackLineEdit = new QLineEdit(fmtbox);
+	QString formatToolTip = ImportParser::getFormatToolTip();
+	QCM_setToolTip(m_headerLineEdit, formatToolTip);
+	QCM_setToolTip(m_trackLineEdit, formatToolTip);
 	connect(m_formatComboBox, SIGNAL(activated(int)), this, SLOT(setFormatLineEdit(int)));
 #if QT_VERSION >= 0x040000
 	QVBoxLayout* vbox = new QVBoxLayout;
@@ -273,10 +277,13 @@ ImportSelector::ImportSelector(
 	butlayout->setMargin(0);
 	butlayout->setSpacing(6);
 	m_fileButton = new QPushButton(i18n("From F&ile"), butbox);
+	m_fileButton->setAutoDefault(false);
 	butlayout->addWidget(m_fileButton);
 	m_clipButton = new QPushButton(i18n("From Clip&board"), butbox);
+	m_clipButton->setAutoDefault(false);
 	butlayout->addWidget(m_clipButton);
 	m_serverButton = new QPushButton(i18n("&From Server:"), butbox);
+	m_serverButton->setAutoDefault(false);
 	butlayout->addWidget(m_serverButton);
 	m_serverComboBox = new QComboBox(butbox);
 	m_serverComboBox->setEditable(false);
@@ -390,6 +397,19 @@ void ImportSelector::setImportServer(ImportConfig::ImportServer server)
 	m_serverComboBox->QCM_setCurrentIndex(server);
 }
 
+/**
+ * Set the format combo box and line edits from the configuration.
+ */
+void ImportSelector::setFormatFromConfig()
+{
+	m_formatHeaders = Kid3App::s_genCfg.m_importFormatHeaders;
+	m_formatTracks = Kid3App::s_genCfg.m_importFormatTracks;
+	m_formatComboBox->clear();
+	m_formatComboBox->QCM_addItems(Kid3App::s_genCfg.m_importFormatNames);
+	m_formatComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_importFormatIdx);
+	setFormatLineEdit(Kid3App::s_genCfg.m_importFormatIdx);
+}
+
 /**
  * Clear dialog data.
  */
@@ -403,12 +423,7 @@ void ImportSelector::clear()
 	m_serverComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_importServer);
 	m_destComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_importDest);
 
-	m_formatHeaders = Kid3App::s_genCfg.m_importFormatHeaders;
-	m_formatTracks = Kid3App::s_genCfg.m_importFormatTracks;
-	m_formatComboBox->clear();
-	m_formatComboBox->QCM_addItems(Kid3App::s_genCfg.m_importFormatNames);
-	m_formatComboBox->QCM_setCurrentIndex(Kid3App::s_genCfg.m_importFormatIdx);
-	setFormatLineEdit(Kid3App::s_genCfg.m_importFormatIdx);
+	setFormatFromConfig();
 
 	m_mismatchCheckBox->setChecked(Kid3App::s_genCfg.m_enableTimeDifferenceCheck);
 	m_maxDiffSpinBox->setValue(Kid3App::s_genCfg.m_maxTimeDifference);
@@ -431,17 +446,14 @@ bool ImportSelector::parseHeader(StandardTags& st)
 }
 
 /**
- * Let user select file, assign file contents to text and preview in
- * table.
+ * Import from a file.
+ *
+ * @param fn file name
+ *
+ * @return true if ok.
  */
-void ImportSelector::fromFile()
+bool ImportSelector::importFromFile(const QString& fn)
 {
-	QString fn =
-#ifdef CONFIG_USE_KDE
-		KFileDialog::getOpenFileName(getImportDir(), QString::null, this);
-#else
-		QFileDialog::QCM_getOpenFileName(this, getImportDir());
-#endif
 	if (!fn.isEmpty()) {
 		QFile file(fn);
 		if (file.open(QCM_ReadOnly)) {
@@ -459,8 +471,25 @@ void ImportSelector::fromFile()
 				showPreview();
 			}
 			file.close();
+			return true;
 		}
 	}
+	return false;
+}
+
+/**
+ * Let user select file, assign file contents to text and preview in
+ * table.
+ */
+void ImportSelector::fromFile()
+{
+	importFromFile(
+#ifdef CONFIG_USE_KDE
+		KFileDialog::getOpenFileName(getImportDir(), QString::null, this)
+#else
+		QFileDialog::QCM_getOpenFileName(this, getImportDir())
+#endif
+		);
 }
 
 /**
@@ -590,8 +619,13 @@ void ImportSelector::fromDiscogs()
  */
 void ImportSelector::setFormatLineEdit(int index)
 {
-	m_headerLineEdit->setText(m_formatHeaders[index]);
-	m_trackLineEdit->setText(m_formatTracks[index]);
+	if (index < static_cast<int>(m_formatHeaders.size())) {
+		m_headerLineEdit->setText(m_formatHeaders[index]);
+		m_trackLineEdit->setText(m_formatTracks[index]);
+	} else {
+		m_headerLineEdit->clear();
+		m_trackLineEdit->clear();
+	}
 }
 
 /**
@@ -868,9 +902,22 @@ ImportConfig::ImportDestination ImportSelector::getDestination()
 }
 
 /**
- * Save the local settings to the configuration.
+ * Set import destination.
+ *
+ * @param dest DestV1, DestV2 or DestV1V2 for ID3v1, ID3v2 or both
  */
-void ImportSelector::saveConfig()
+void ImportSelector::setDestination(ImportConfig::ImportDestination dest)
+{
+	m_destComboBox->QCM_setCurrentIndex(dest);
+}
+
+/**
+ * Save the local settings to the configuration.
+ *
+ * @param width  window width
+ * @param height window height
+ */
+void ImportSelector::saveConfig(int width, int height)
 {
 	Kid3App::s_genCfg.m_importDest = static_cast<ImportConfig::ImportDestination>(
 		m_destComboBox->QCM_currentIndex());
@@ -878,12 +925,23 @@ void ImportSelector::saveConfig()
 	Kid3App::s_genCfg.m_importServer = static_cast<ImportConfig::ImportServer>(
 		m_serverComboBox->QCM_currentIndex());
 	Kid3App::s_genCfg.m_importFormatIdx = m_formatComboBox->QCM_currentIndex();
-	Kid3App::s_genCfg.m_importFormatNames[Kid3App::s_genCfg.m_importFormatIdx] = m_formatComboBox->currentText();
-	Kid3App::s_genCfg.m_importFormatHeaders[Kid3App::s_genCfg.m_importFormatIdx] = m_headerLineEdit->text();
-	Kid3App::s_genCfg.m_importFormatTracks[Kid3App::s_genCfg.m_importFormatIdx] = m_trackLineEdit->text();
+	if (Kid3App::s_genCfg.m_importFormatIdx < static_cast<int>(Kid3App::s_genCfg.m_importFormatNames.size())) {
+		Kid3App::s_genCfg.m_importFormatNames[Kid3App::s_genCfg.m_importFormatIdx] = m_formatComboBox->currentText();
+		Kid3App::s_genCfg.m_importFormatHeaders[Kid3App::s_genCfg.m_importFormatIdx] = m_headerLineEdit->text();
+		Kid3App::s_genCfg.m_importFormatTracks[Kid3App::s_genCfg.m_importFormatIdx] = m_trackLineEdit->text();
+	} else {
+		Kid3App::s_genCfg.m_importFormatIdx = Kid3App::s_genCfg.m_importFormatNames.size();
+		Kid3App::s_genCfg.m_importFormatNames.append(m_formatComboBox->currentText());
+		Kid3App::s_genCfg.m_importFormatHeaders.append(m_headerLineEdit->text());
+		Kid3App::s_genCfg.m_importFormatTracks.append(m_trackLineEdit->text());
+	}
 	getTimeDifferenceCheck(Kid3App::s_genCfg.m_enableTimeDifferenceCheck,
 												 Kid3App::s_genCfg.m_maxTimeDifference);
 
+	Kid3App::s_genCfg.m_importWindowWidth = width;
+	Kid3App::s_genCfg.m_importWindowHeight = height;
+
+	setFormatFromConfig();
 }
 
 /**
@@ -930,7 +988,7 @@ void ImportSelector::maxDiffChanged() {
 /**
  * Move a table row.
  *
- * @param section not used
+ * The first parameter @a section is not used.
  * @param fromIndex index of position moved from
  * @param fromIndex index of position moved to
  */
