@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 17 Sep 2003
  *
- * Copyright (C) 2003-2007  Urs Fleisch
+ * Copyright (C) 2003-2008  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -116,6 +116,13 @@ public:
 	 */
 	ImportConfig::ImportDestination getDestination();
 
+	/**
+	 * Set import destination.
+	 *
+	 * @param dest DestV1, DestV2 or DestV1V2 for ID3v1, ID3v2 or both
+	 */
+	void setDestination(ImportConfig::ImportDestination dest);
+
 	/**
 	 * Get list with track durations.
 	 *
@@ -132,6 +139,23 @@ public:
 	 */ 
 	void getTimeDifferenceCheck(bool& enable, int& maxDiff) const;
 
+	/**
+	 * Import from a file.
+	 *
+	 * @param fn file name
+	 *
+	 * @return true if ok.
+	 */
+	bool importFromFile(const QString& fn);
+
+	/**
+	 * Save the local settings to the configuration.
+	 *
+	 * @param width  window width
+	 * @param height window height
+	 */
+	void saveConfig(int width, int height);
+
 	/**
 	 * List with line formats.
 	 * The following codes are used before the () expressions.
@@ -240,11 +264,6 @@ public slots:
 	 */
 	void matchWithTitle();
 
-	/**
-	 * Save the local settings to the configuration.
-	 */
-	void saveConfig();
-
 private:
 	enum TabColumn {
 		LengthColumn, TrackColumn, TitleColumn, ArtistColumn,
@@ -264,6 +283,11 @@ private:
 	 */
 	bool updateTrackData(ImportSource impSrc);
 
+	/**
+	 * Set the format combo box and line edits from the configuration.
+	 */
+	void setFormatFromConfig();
+
 	/** From File button */
 	QPushButton* m_fileButton;
 	/** From Clipboard button */
