@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 9 Jan 2003
  *
- * Copyright (C) 2003-2007  Urs Fleisch
+ * Copyright (C) 2003-2008  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -34,6 +34,7 @@
 #include <qmessagebox.h>
 #include <qpushbutton.h>
 #include <qprogressbar.h>
+#include <qimage.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
 #include <QCloseEvent>
@@ -43,6 +44,10 @@
 #else
 #include <qlayout.h>
 #endif
+#ifdef HAVE_QTDBUS
+#include <QDBusConnection>
+#include "scriptinterface.h"
+#endif
 
 #ifdef CONFIG_USE_KDE
 #include <kapplication.h>
@@ -82,10 +87,12 @@
 #include "importdialog.h"
 #include "exportdialog.h"
 #include "numbertracksdialog.h"
+#include "filterdialog.h"
 #include "standardtags.h"
 #include "rendirdialog.h"
 #include "filelistitem.h"
 #include "dirlist.h"
+#include "pictureframe.h"
 #ifdef HAVE_ID3LIB
 #include "mp3file.h"
 #endif
@@ -224,17 +231,17 @@ FreedbConfig Kid3App::s_freedbCfg("Freedb");
 FreedbConfig Kid3App::s_trackTypeCfg("TrackType");
 DiscogsConfig Kid3App::s_discogsCfg("Discogs");
 MusicBrainzConfig Kid3App::s_musicBrainzCfg("MusicBrainz");
+FilterConfig Kid3App::s_filterCfg("Filter");
 
 /** Current directory */
 QString Kid3App::s_dirName;
 
 /**
  * Constructor.
- *
- * @param name name
  */
 Kid3App::Kid3App() :
-	m_importDialog(0), m_exportDialog(0), m_numberTracksDialog(0)
+	m_importDialog(0), m_exportDialog(0), m_renDirDialog(0),
+	m_numberTracksDialog(0), m_filterDialog(0)
 {
 #ifdef CONFIG_USE_KDE
 #if KDE_VERSION >= 0x035c00
@@ -259,10 +266,28 @@ Kid3App::Kid3App() :
 #endif
 	readFontAndStyleOptions();
 #endif
+#ifdef HAVE_QTDBUS
+	if (QDBusConnection::sessionBus().isConnected()) {
+		QString serviceName("net.sourceforge.kid3");
+    QDBusConnection::sessionBus().registerService(serviceName);
+#ifndef CONFIG_USE_KDE
+		serviceName += '-';
+		serviceName += QString::number(::getpid());
+    QDBusConnection::sessionBus().registerService(serviceName);
+#endif
+		new ScriptInterface(this);
+		if (!QDBusConnection::sessionBus().registerObject("/Kid3", this)) {
+			qWarning("Registering D-Bus object failed");
+		}
+	} else {
+		qWarning("Cannot connect to the D-BUS session bus.");
+	}
+#endif
 
 	initFileTypes();
 	initStatusBar();
 	setModified(false);
+	setFiltered(false);
 	initView();
 	initActions();
 	s_fnFormatCfg.setAsFilenameFormatter();
@@ -278,7 +303,9 @@ Kid3App::Kid3App() :
 Kid3App::~Kid3App()
 {
 	delete m_importDialog;
+	delete m_renDirDialog;
 	delete m_numberTracksDialog;
+	delete m_filterDialog;
 #ifndef CONFIG_USE_KDE
 	delete s_helpBrowser;
 	s_helpBrowser = 0;
@@ -396,6 +423,10 @@ void Kid3App::initActions()
 		    i18n("&Number Tracks..."), this,
 		    SLOT(slotNumberTracks()), actionCollection(),
 		    "number_tracks");
+	KCM_KAction(toolsFilter,
+		    i18n("F&ilter..."), this,
+		    SLOT(slotFilter()), actionCollection(),
+		    "filter");
 #ifdef HAVE_TAGLIB
 	KCM_KAction(toolsConvertToId3v24,
 		    i18n("Convert ID3v2.3 to ID3v2.&4"), this,
@@ -662,6 +693,13 @@ void Kid3App::initActions()
 		connect(toolsNumberTracks, QCM_SIGNAL_triggered,
 			this, SLOT(slotNumberTracks()));
 	}
+	QAction* toolsFilter = new QAction(this);
+	if (toolsFilter) {
+		toolsFilter->setStatusTip(i18n("Filter"));
+		toolsFilter->QCM_setMenuText(i18n("F&ilter..."));
+		connect(toolsFilter, QCM_SIGNAL_triggered,
+			this, SLOT(slotFilter()));
+	}
 #ifdef HAVE_TAGLIB
 	QAction* toolsConvertToId3v24 = new QAction(this);
 	if (toolsConvertToId3v24) {
@@ -750,6 +788,7 @@ void Kid3App::initActions()
 		QCM_addAction(toolsMenu, toolsApplyId3Format);
 		QCM_addAction(toolsMenu, toolsRenameDirectory);
 		QCM_addAction(toolsMenu, toolsNumberTracks);
+		QCM_addAction(toolsMenu, toolsFilter);
 #ifdef HAVE_TAGLIB
 		QCM_addAction(toolsMenu, toolsConvertToId3v24);
 #endif
@@ -819,20 +858,27 @@ void Kid3App::initView()
 /**
  * Open directory.
  *
- * @param dir     directory or file path
- * @param confirm if true ask if there are unsaved changes
+ * @param dir       directory or file path
+ * @param confirm   if true ask if there are unsaved changes
+ * @param fileCheck if true and dir in not directory, only open directory
+ *                  if dir is a valid file path
+ *
+ * @return true if ok.
  */
-void Kid3App::openDirectory(QString dir, bool confirm)
+bool Kid3App::openDirectory(QString dir, bool confirm, bool fileCheck)
 {
 	if (confirm && !saveModified()) {
-		return;
+		return false;
 	}
 	if (dir.isNull() || dir.isEmpty()) {
-		return;
+		return false;
 	}
 	QFileInfo file(dir);
 	QString fileName;
 	if (!file.isDir()) {
+		if (fileCheck && !file.isFile()) {
+			return false;
+		}
 #if QT_VERSION >= 0x040000
 		dir = file.dir().path();
 #else
@@ -843,9 +889,11 @@ void Kid3App::openDirectory(QString dir, bool confirm)
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 	slotStatusMsg(i18n("Opening directory..."));
-	if (m_view->readFileList(dir, fileName)) {
+	bool ok = m_view->readFileList(dir, fileName);
+	if (ok) {
 		m_view->readDirectoryList(dir);
 		setModified(false);
+		setFiltered(false);
 #ifdef CONFIG_USE_KDE
 #if KDE_VERSION >= 0x035c00
 		KUrl url;
@@ -862,6 +910,7 @@ void Kid3App::openDirectory(QString dir, bool confirm)
 	}
 	slotStatusMsg(i18n("Ready."));
 	QApplication::restoreOverrideCursor();
+	return ok;
 }
 
 /**
@@ -888,6 +937,7 @@ void Kid3App::saveOptions()
 	s_freedbCfg.writeToConfig(m_config);
 	s_trackTypeCfg.writeToConfig(m_config);
 	s_discogsCfg.writeToConfig(m_config);
+	s_filterCfg.writeToConfig(m_config);
 #ifdef HAVE_TUNEPIMP
 	s_musicBrainzCfg.writeToConfig(m_config);
 #endif
@@ -922,6 +972,7 @@ void Kid3App::readOptions()
 		s_trackTypeCfg.m_server = "tracktype.org:80"; // replace default
 	}
 	s_discogsCfg.readFromConfig(m_config);
+	s_filterCfg.readFromConfig(m_config);
 #ifdef HAVE_TUNEPIMP
 	s_musicBrainzCfg.readFromConfig(m_config);
 #endif
@@ -963,7 +1014,7 @@ void Kid3App::saveProperties(KConfigGroup& cfg)
  *
  * @param cfg application configuration
  */
-void Kid3App::readProperties(KConfigGroup& cfg)
+void Kid3App::readProperties(const KConfigGroup& cfg)
 {
 	openDirectory(cfg.readEntry("dirname", ""));
 }
@@ -1032,10 +1083,20 @@ void Kid3App::readFontAndStyleOptions()
 /**
  * Save all changed files.
  *
- * @return true
+ * @param updateGui true to update GUI (controls, status, cursor)
+ * @param errStr    if not 0, the error string is returned here and
+ *                  no dialog is displayed
+ *
+ * @return true if ok.
  */
-bool Kid3App::saveDirectory()
+bool Kid3App::saveDirectory(bool updateGui, QString* errStr)
 {
+	if (updateGui) {
+		updateCurrentSelection();
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+		slotStatusMsg(i18n("Saving directory..."));
+	}
+
 	QStringList errorFiles;
 	int numFiles = 0, totalFiles = 0;
 	FileListItem* mp3file = m_view->firstFile();
@@ -1083,20 +1144,30 @@ bool Kid3App::saveDirectory()
 	delete progress;
 	updateModificationState();
 	if (!errorFiles.empty()) {
+		if (errStr) {
+			*errStr = errorFiles.join("\n");
+		} else {
 #ifdef CONFIG_USE_KDE
-		KMessageBox::errorList(
-			0, i18n("Error while writing file:\n"),
-			errorFiles,
-			i18n("File Error"));
+			KMessageBox::errorList(
+				0, i18n("Error while writing file:\n"),
+				errorFiles,
+				i18n("File Error"));
 #else
-		QMessageBox::warning(
-			0, i18n("File Error"),
-			i18n("Error while writing file:\n") +
-			errorFiles.join("\n"),
-			QMessageBox::Ok, QCM_NoButton);
+			QMessageBox::warning(
+				0, i18n("File Error"),
+				i18n("Error while writing file:\n") +
+				errorFiles.join("\n"),
+				QMessageBox::Ok, QCM_NoButton);
 #endif
+		}
 	}
-	return true;
+
+	if (updateGui) {
+		slotStatusMsg(i18n("Ready."));
+		QApplication::restoreOverrideCursor();
+		updateGuiControls();
+	}
+	return errorFiles.empty();
 }
 
 /**
@@ -1442,14 +1513,7 @@ void Kid3App::slotFileRevert()
  */
 void Kid3App::slotFileSave()
 {
-	updateCurrentSelection();
-	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-	slotStatusMsg(i18n("Saving directory..."));
-
-	saveDirectory();
-	slotStatusMsg(i18n("Ready."));
-	QApplication::restoreOverrideCursor();
-	updateGuiControls();
+	saveDirectory(true);
 }
 
 /**
@@ -1590,7 +1654,7 @@ void Kid3App::slotHelpAbout()
 	QMessageBox::about(
 		(Kid3App*)parent(), "Kid3",
 		"Kid3 " VERSION
-		"\n(c) 2003-2007 Urs Fleisch\nufleisch@users.sourceforge.net");
+		"\n(c) 2003-2008 Urs Fleisch\nufleisch@users.sourceforge.net");
 }
 
 /**
@@ -1622,18 +1686,21 @@ void Kid3App::slotStatusMsg(const QString& text)
 
 /**
  * Create playlist.
+ *
+ * @return true if ok.
  */
-void Kid3App::slotCreatePlaylist()
+bool Kid3App::slotCreatePlaylist()
 {
 	FileListItem* mp3file = m_view->firstFileInDir();
-	if (!(mp3file && mp3file->getFile())) return;
+	if (!(mp3file && mp3file->getFile())) return false;
 	QDir dir(mp3file->getFile()->getDirname());
 	QString dirname = dir.QCM_absolutePath();
 	QString fn = dirname + QDir::separator() + dir.dirName() + ".m3u";
 	QFile file(fn);
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 	slotStatusMsg(i18n("Creating playlist..."));
-	if (file.open(QCM_WriteOnly)) {
+	bool ok = file.open(QCM_WriteOnly);
+	if (ok) {
 		QTextStream stream(&file);
 		while (mp3file != 0) {
 			stream << mp3file->getFile()->getFilename() << "\n";
@@ -1643,6 +1710,7 @@ void Kid3App::slotCreatePlaylist()
 	}
 	slotStatusMsg(i18n("Ready."));
 	QApplication::restoreOverrideCursor();
+	return ok;
 }
 
 /**
@@ -1653,6 +1721,7 @@ void Kid3App::setupImportDialog()
 	m_trackDataList.clear();
 	FileListItem* mp3file = m_view->firstFileInDir();
 	bool firstTrack = true;
+	bool tag1Supported = true;
 	while (mp3file != 0) {
 		mp3file->getFile()->readTags(false);
 		if (firstTrack) {
@@ -1664,6 +1733,7 @@ void Kid3App::setupImportDialog()
 			m_trackDataList.m_artist = st.artist;
 			m_trackDataList.m_album = st.album;
 			firstTrack = false;
+			tag1Supported = mp3file->getFile()->isTagV1Supported();
 		}
 		m_trackDataList.push_back(ImportTrackData(mp3file->getFile()->getAbsFilename(),
                                             mp3file->getFile()->getDuration()));
@@ -1677,9 +1747,65 @@ void Kid3App::setupImportDialog()
 	}
 	if (m_importDialog) {
 		m_importDialog->clear();
+		if (!tag1Supported &&
+				m_importDialog->getDestination() == ImportConfig::DestV1) {
+			m_importDialog->setDestination(ImportConfig::DestV2);
+		}
 	}
 }
 
+/**
+ * Import tags from the import dialog.
+ *
+ * @param destV1 true to set tag 1
+ * @param destV2 true to set tag 2
+ */
+void Kid3App::getTagsFromImportDialog(bool destV1, bool destV2)
+{
+	slotStatusMsg(i18n("Import..."));
+	ImportTrackDataVector::const_iterator it = m_trackDataList.begin();
+	StandardTags st1, st2;
+	StandardTagsFilter flt(destV1 ?
+												 m_view->getFilterFromID3V1() :
+												 m_view->getFilterFromID3V2());
+	bool no_selection = m_view->numFilesSelected() == 0;
+	FileListItem* mp3file = m_view->firstFileInDir();
+	while (mp3file != 0) {
+		mp3file->getFile()->readTags(false);
+		if (destV1) mp3file->getFile()->getStandardTagsV1(&st1);
+		if (destV2) mp3file->getFile()->getStandardTagsV2(&st2);
+		if (it != m_trackDataList.end()) {
+			if (destV1) (*it).copyActiveTags(st1);
+			if (destV2) (*it).copyActiveTags(st2);
+			++it;
+		} else {
+			break;
+		}
+		if (destV1) {
+			formatStandardTagsIfEnabled(&st1);
+			mp3file->getFile()->setStandardTagsV1(&st1, flt);
+		}
+		if (destV2) {
+			formatStandardTagsIfEnabled(&st2);
+			mp3file->getFile()->setStandardTagsV2(&st2, flt);
+		}
+		mp3file = m_view->nextFileInDir();
+	}
+	if (!no_selection) {
+		m_view->frameTableV1()->frames().clear();
+		m_view->frameTableV1()->framesToTable();
+		m_view->frameTableV2()->frames().clear();
+		m_view->frameTableV2()->framesToTable();
+		m_view->setFilenameEditEnabled(false);
+		fileSelected();
+	}
+	else {
+		updateModificationState();
+	}
+	slotStatusMsg(i18n("Ready."));
+	QApplication::restoreOverrideCursor();
+}
+
 /**
  * Execute the import dialog.
  */
@@ -1687,55 +1813,37 @@ void Kid3App::execImportDialog()
 {
 	if (m_importDialog &&
 			m_importDialog->exec() == QDialog::Accepted) {
-		slotStatusMsg(i18n("Import..."));
-		ImportTrackDataVector::const_iterator it = m_trackDataList.begin();
-		StandardTags st1, st2;
 		bool destV1 = m_importDialog->getDestination() == ImportConfig::DestV1 ||
 		              m_importDialog->getDestination() == ImportConfig::DestV1V2;
 		bool destV2 = m_importDialog->getDestination() == ImportConfig::DestV2 ||
 		              m_importDialog->getDestination() == ImportConfig::DestV1V2;
-		StandardTagsFilter flt(destV1 ?
-													 m_view->getFilterFromID3V1() :
-													 m_view->getFilterFromID3V2());
-		bool no_selection = m_view->numFilesSelected() == 0;
-		FileListItem* mp3file = m_view->firstFileInDir();
-		while (mp3file != 0) {
-			mp3file->getFile()->readTags(false);
-			if (destV1) mp3file->getFile()->getStandardTagsV1(&st1);
-			if (destV2) mp3file->getFile()->getStandardTagsV2(&st2);
-			if (it != m_trackDataList.end()) {
-				if (destV1) (*it).copyActiveTags(st1);
-				if (destV2) (*it).copyActiveTags(st2);
-				++it;
-			} else {
-				break;
-			}
-			if (destV1) {
-				formatStandardTagsIfEnabled(&st1);
-				mp3file->getFile()->setStandardTagsV1(&st1, flt);
-			}
-			if (destV2) {
-				formatStandardTagsIfEnabled(&st2);
-				mp3file->getFile()->setStandardTagsV2(&st2, flt);
-			}
-			mp3file = m_view->nextFileInDir();
-		}
-		if (!no_selection) {
-			m_view->frameTableV1()->frames().clear();
-			m_view->frameTableV1()->framesToTable();
-			m_view->frameTableV2()->frames().clear();
-			m_view->frameTableV2()->framesToTable();
-			m_view->setFilenameEditEnabled(false);
-			fileSelected();
-		}
-		else {
-			updateModificationState();
-		}
-		slotStatusMsg(i18n("Ready."));
-		QApplication::restoreOverrideCursor();
+		getTagsFromImportDialog(destV1, destV2);
 	}
 }
 
+/**
+ * Import.
+ *
+ * @param tagMask tag mask (bit 0 for tag 1, bit 1 for tag 2)
+ * @param path    path of file
+ * @param fmtIdx  index of format
+ *
+ * @return true if ok.
+ */
+bool Kid3App::importTags(int tagMask, const QString& path, int fmtIdx)
+{
+	setupImportDialog();
+	if (m_importDialog) {
+		m_importDialog->setAutoStartSubDialog(ImportDialog::ASD_None);
+		m_importDialog->setFormatLineEdit(fmtIdx);
+		if (m_importDialog->importFromFile(path)) {
+			getTagsFromImportDialog((tagMask & 1) != 0, (tagMask & 2) != 0);
+			return true;
+		}
+	}
+	return false;
+}
+
 /**
  * Import.
  */
@@ -1843,6 +1951,33 @@ void Kid3App::setExportData(int src)
 	}
 }
 
+/**
+ * Export.
+ *
+ * @param tagNr  tag number (1 or 2)
+ * @param path   path of file
+ * @param fmtIdx index of format
+ *
+ * @return true if ok.
+ */
+bool Kid3App::exportTags(int tagNr, const QString& path, int fmtIdx)
+{
+	bool ok = false;
+	m_exportDialog = new ExportDialog(0);
+	if (m_exportDialog) {
+		m_exportDialog->readConfig();
+		m_exportDialog->setFormatLineEdit(fmtIdx);
+		setExportData(tagNr == 2 ?
+									ExportDialog::SrcV2 : ExportDialog::SrcV1);
+		connect(m_exportDialog, SIGNAL(exportDataRequested(int)),
+						this, SLOT(setExportData(int)));
+		ok = m_exportDialog->exportToFile(path);
+		delete m_exportDialog;
+		m_exportDialog = 0;
+	}
+	return ok;
+}
+
 /**
  * Export.
  */
@@ -2036,44 +2171,197 @@ void Kid3App::slotApplyId3Format()
 	updateGuiControls();
 }
 
+#if defined HAVE_ID3LIB && defined HAVE_TAGLIB
+/**
+ * Read file with TagLib if it has an ID3v2.4 tag.
+ *
+ * @param item       file list item, can be updated
+ * @param taggedFile tagged file
+ *
+ * @return tagged file (can be new TagLibFile).
+ */
+static TaggedFile* readWithTagLibIfId3V24(FileListItem* item,
+																					TaggedFile* taggedFile)
+{
+	if (dynamic_cast<Mp3File*>(taggedFile) != 0 &&
+			!taggedFile->isChanged() &&
+			taggedFile->isTagInformationRead() && taggedFile->hasTagV2() &&
+			taggedFile->getTagFormatV2() == QString::null) {
+		TagLibFile* tagLibFile;
+		if ((tagLibFile = new TagLibFile(
+					 taggedFile->getDirInfo(),
+					 taggedFile->getFilename())) != 0) {
+			item->setFile(tagLibFile);
+			taggedFile = tagLibFile;
+			taggedFile->readTags(false);
+		}
+	}
+	return taggedFile;
+}
+#endif
+
+/**
+ * Schedule actions to rename a directory.
+ */
+void Kid3App::scheduleRenameActions()
+{
+	if (m_renDirDialog) {
+		m_renDirDialog->clearActions();
+		TaggedFile* taggedFile;
+		FileListItem* item = m_view->firstFileOrDir();
+		while (item) {
+			if (item->getDirInfo()) {
+#if QT_VERSION >= 0x040000
+				item->setExpanded(true);
+#else
+				item->setOpen(true);
+#endif
+			} else if ((taggedFile = item->getFile()) != 0) {
+				taggedFile->readTags(false);
+#if defined HAVE_ID3LIB && defined HAVE_TAGLIB
+				taggedFile = readWithTagLibIfId3V24(item, taggedFile);
+#endif
+				m_renDirDialog->scheduleAction(taggedFile);
+			}
+			item = m_view->nextFileOrDir();
+#ifdef CONFIG_USE_KDE
+			kapp->processEvents();
+#else
+			qApp->processEvents();
+#endif
+			if (m_renDirDialog->getAbortFlag()) {
+				break;
+			}
+		}
+	}
+}
+
+/**
+ * Set the directory name from the tags.
+ * The directory must not have modified files.
+ *
+ * @param tagMask tag mask (bit 0 for tag 1, bit 1 for tag 2)
+ * @param format  directory name format
+ * @param create  true to create, false to rename
+ * @param errStr  if not 0, a string describing the error is returned here
+ *
+ * @return true if ok.
+ */
+bool Kid3App::renameDirectory(int tagMask, const QString& format,
+															bool create, QString* errStr)
+{
+	bool ok = false;
+	if (!isModified() && m_view->firstFileInDir()) {
+		if (!m_renDirDialog) {
+			m_renDirDialog = new RenDirDialog(0);
+			connect(m_renDirDialog, SIGNAL(actionSchedulingRequested()),
+							this, SLOT(scheduleRenameActions()));
+		}
+		if (m_renDirDialog) {
+			m_renDirDialog->startDialog(m_view->firstFileInDir()->getFile());
+			m_renDirDialog->setTagSource(tagMask);
+			m_renDirDialog->setDirectoryFormat(format);
+			m_renDirDialog->setAction(create);
+			scheduleRenameActions();
+			openDirectory(getDirName());
+			QString errorMsg;
+			m_renDirDialog->performActions(&errorMsg);
+			openDirectory(m_renDirDialog->getNewDirname());
+			ok = errorMsg.isEmpty();
+			if (errStr) {
+				*errStr = errorMsg;
+			}
+		}
+	}
+	return ok;
+}
+
 /**
  * Rename directory.
  */
 void Kid3App::slotRenameDirectory()
 {
-	if (saveModified() && m_view->firstFileInDir()) {
-		QString caption(i18n("Rename Directory"));
-		RenDirDialog* dialog =
-			new RenDirDialog(NULL, caption, m_view->firstFileInDir()->getFile());
-		if (dialog) {
-			if (dialog->exec() == QDialog::Accepted) {
-				FileListItem* mp3file = m_view->firstFileInDir();
-				QString errorMsg;
-				bool again = false;
-				while (mp3file &&
-					   dialog->performAction(mp3file->getFile(), again, &errorMsg)) {
-					mp3file = m_view->nextFileInDir();
+	if (saveModified()) {
+		if (!m_renDirDialog) {
+			m_renDirDialog = new RenDirDialog(0);
+			connect(m_renDirDialog, SIGNAL(actionSchedulingRequested()),
+							this, SLOT(scheduleRenameActions()));
+		}
+		if (m_renDirDialog) {
+			FileListItem* item = m_view->currentFile();
+			if (item && item->isSelected()) {
+				TaggedFile* taggedFile;
+				const DirInfo* dirInfo = item->getDirInfo();
+				if (dirInfo) {
+#if QT_VERSION >= 0x040000
+					item->setExpanded(true);
+#else
+					item->setOpen(true);
+#endif
+				} else if ((taggedFile = item->getFile()) != 0) {
+					dirInfo = taggedFile->getDirInfo();
 				}
-				openDirectory(dialog->getNewDirname());
-				if (again) {
-					mp3file = m_view->firstFileInDir();
-					while (mp3file &&
-						   dialog->performAction(mp3file->getFile(), again, &errorMsg)) {
-						mp3file = m_view->nextFileInDir();
-					}
-					openDirectory(dialog->getNewDirname());
-				}
-				if (!errorMsg.isEmpty()) {
-					QMessageBox::warning(0, i18n("File Error"),
-										 i18n("Error while renaming:\n") +
-										 errorMsg,
-										 QMessageBox::Ok, QCM_NoButton);
+				if (dirInfo) {
+					openDirectory(dirInfo->getDirname());
 				}
 			}
+			item = m_view->firstFileInDir();
+			if (item) {
+				m_renDirDialog->startDialog(item->getFile());
+			} else {
+				m_renDirDialog->startDialog(0, getDirName());
+			}
+			if (m_renDirDialog->exec() == QDialog::Accepted) {
+				openDirectory(getDirName());
+				QString errorMsg;
+				m_renDirDialog->performActions(&errorMsg);
+				openDirectory(m_renDirDialog->getNewDirname());
+				if (!errorMsg.isEmpty()) {
+					QMessageBox::warning(0, i18n("File Error"),
+															 i18n("Error while renaming:\n") +
+															 errorMsg,
+															 QMessageBox::Ok, QCM_NoButton);
+				}
+			}
+
 		}
 	}
 }
 
+/**
+ * Number tracks in selected files of directory.
+ *
+ * @param nr start number
+ * @param destV1 true to set numbers in tag 1
+ * @param destV2 true to set numbers in tag 2
+ */
+void Kid3App::numberTracks(int nr, bool destV1, bool destV2)
+{
+	updateCurrentSelection();
+	FileListItem* mp3file = m_view->firstFileInDir();
+	bool no_selection = m_view->numFilesSelected() == 0;
+	while (mp3file != 0) {
+		if (no_selection || mp3file->isInSelection()) {
+			mp3file->getFile()->readTags(false);
+			if (destV1) {
+				int oldnr = mp3file->getFile()->getTrackNumV1();
+				if (nr != oldnr) {
+					mp3file->getFile()->setTrackNumV1(nr);
+				}
+			}
+			if (destV2) {
+				int oldnr = mp3file->getFile()->getTrackNumV2();
+				if (nr != oldnr) {
+					mp3file->getFile()->setTrackNumV2(nr);
+				}
+			}
+			++nr;
+		}
+		mp3file = m_view->nextFileInDir();
+	}
+	updateGuiControls();
+}
+
 /**
  * Number tracks.
  */
@@ -2091,30 +2379,109 @@ void Kid3App::slotNumberTracks()
 			bool destV2 =
 				m_numberTracksDialog->getDestination() == NumberTracksDialog::DestV2 ||
 				m_numberTracksDialog->getDestination() == NumberTracksDialog::DestV1V2;
+			numberTracks(nr, destV1, destV2);
+		}
+	}
+}
 
-			updateCurrentSelection();
-			FileListItem* mp3file = m_view->firstFileInDir();
-			bool no_selection = m_view->numFilesSelected() == 0;
-			while (mp3file != 0) {
-				if (no_selection || mp3file->isInSelection()) {
-					mp3file->getFile()->readTags(false);
-					if (destV1) {
-						int oldnr = mp3file->getFile()->getTrackNumV1();
-						if (nr != oldnr) {
-							mp3file->getFile()->setTrackNumV1(nr);
-						}
+/**
+ * Apply a file filter.
+ *
+ * @param fileFilter filter to apply.
+ */
+void Kid3App::applyFilter(FileFilter& fileFilter)
+{
+	if (isFiltered()) {
+		const DirInfo* dirInfo = m_view->getDirInfo();
+		if (dirInfo) {
+			m_view->readFileList(dirInfo->getDirname());
+		}
+		setFiltered(false);
+	}
+
+	if (m_filterDialog) {
+		m_filterDialog->clearAbortFlag();
+	}
+
+	FileListItem* item = m_view->firstFileOrDir();
+#if QT_VERSION >= 0x040000
+	QList<FileListItem*> itemsToDelete;
+#else
+	QValueList<FileListItem*> itemsToDelete;
+#endif
+	while (item != 0) {
+		if (item->getDirInfo()) {
+#if QT_VERSION >= 0x040000
+			item->setExpanded(true);
+#else
+			item->setOpen(true);
+#endif
+		} else {
+			TaggedFile* taggedFile = item->getFile();
+			if (taggedFile) {
+				taggedFile->readTags(false);
+#if defined HAVE_ID3LIB && defined HAVE_TAGLIB
+				taggedFile = readWithTagLibIfId3V24(item, taggedFile);
+#endif
+				bool ok;
+				bool pass = fileFilter.filter(*taggedFile, &ok);
+				if (!ok) {
+					if (m_filterDialog) {
+						m_filterDialog->showInformation("parse error");
 					}
-					if (destV2) {
-						int oldnr = mp3file->getFile()->getTrackNumV2();
-						if (nr != oldnr) {
-							mp3file->getFile()->setTrackNumV2(nr);
-						}
-					}
-					++nr;
+					break;
+				}
+				if (m_filterDialog) {
+					m_filterDialog->showInformation(
+						(pass ? QString("+\t") : QString("-\t")) +
+						taggedFile->getFilename());
+				}
+				if (!pass) {
+					itemsToDelete.push_back(item);
 				}
-				mp3file = m_view->nextFileInDir();
 			}
-			updateGuiControls();
+		}
+		item = m_view->nextFileOrDir();
+#ifdef CONFIG_USE_KDE
+		kapp->processEvents();
+#else
+		qApp->processEvents();
+#endif
+		if (m_filterDialog && m_filterDialog->getAbortFlag()) {
+			itemsToDelete.clear();
+			break;
+		}
+	}
+	if (!itemsToDelete.empty()) {
+#if QT_VERSION >= 0x040000
+		qDeleteAll(itemsToDelete);
+#else
+		for (QValueList<FileListItem*>::iterator it = itemsToDelete.begin();
+				 it != itemsToDelete.end();
+				 ++it) {
+			delete *it;
+		}
+#endif
+		setFiltered(true);
+	}
+	updateModificationState();
+}
+
+/**
+ * Filter.
+ */
+void Kid3App::slotFilter()
+{
+	if (saveModified()) {
+		if (!m_filterDialog) {
+			m_filterDialog = new FilterDialog(0);
+			connect(m_filterDialog, SIGNAL(apply(FileFilter&)),
+							this, SLOT(applyFilter(FileFilter&)));
+		}
+		if (m_filterDialog) {
+			s_filterCfg.setFilenameFormat(m_view->getFilenameFormat());
+			m_filterDialog->readConfig();
+			m_filterDialog->exec();
 		}
 	}
 }
@@ -2242,35 +2609,43 @@ void Kid3App::openDrop(QString txt)
 	}
 	QUrl url(txt);
 	if (!url.path().isEmpty()) {
-		QString dir = url.path().QCM_trimmed();
 #if defined _WIN32 || defined WIN32
-		// There seems to be problems with filenames on Win32,
-		// so correct
-		if (dir[0] == '/' && dir[1] == '/' && dir[3] == '|') {
-			dir[3] = ':';
-			dir.remove(0, 2);
-		} else if (dir[0] == '/' && dir[2] == ':') {
-			dir.remove(0, 1);
-		}
+		QString dir = url.toString();
+#else
+		QString dir = url.path().QCM_trimmed();
 #endif
-		updateCurrentSelection();
-		openDirectory(dir, true);
+		if (dir.endsWith(".jpg", QCM_CaseInsensitive) ||
+				dir.endsWith(".png", QCM_CaseInsensitive)) {
+			PictureFrame frame;
+			if (PictureFrame::setDataFromFile(frame, dir)) {
+				QString fileName(dir);
+				int slashPos = fileName.QCM_lastIndexOf('/');
+				if (slashPos != -1) {
+					fileName = fileName.mid(slashPos + 1);
+				}
+				PictureFrame::setMimeTypeFromFileName(frame, fileName);
+				PictureFrame::setDescription(frame, fileName);
+				addFrame(&frame);
+			}
+		} else {
+			updateCurrentSelection();
+			openDirectory(dir, true);
+		}
 	}
 }
 
 /**
- * Set tags in file to tags in GUI controls.
+ * Add picture on drop.
  *
- * @param mp3file file
+ * @param image dropped image.
  */
-void Kid3App::updateTags(TaggedFile* mp3file)
+void Kid3App::dropImage(const QImage& image)
 {
-	m_view->frameTableV1()->tableToFrames();
-	mp3file->setFramesV1(m_view->frameTableV1()->frames());
-	m_view->frameTableV2()->tableToFrames();
-	mp3file->setFramesV2(m_view->frameTableV2()->frames());
-	if (m_view->isFilenameEditEnabled()) {
-		mp3file->setFilename(m_view->getFilename());
+	if (!image.isNull()) {
+		PictureFrame frame;
+		if (PictureFrame::setDataFromImage(frame, image)) {
+			addFrame(&frame);
+		}
 	}
 }
 
@@ -2280,10 +2655,13 @@ void Kid3App::updateTags(TaggedFile* mp3file)
 void Kid3App::updateModificationState()
 {
 	setModified(m_view->updateModificationState());
-#ifdef CONFIG_USE_KDE
-	setCaption(s_dirName, isModified());
-#else
 	QString cap(s_dirName);
+	if (isFiltered()) {
+		cap += i18n(" [filtered]");
+	}
+#ifdef CONFIG_USE_KDE
+	setCaption(cap, isModified());
+#else
 	if (isModified()) {
 		cap += i18n(" [modified]");
 	}
@@ -2300,13 +2678,33 @@ void Kid3App::updateModificationState()
  */
 void Kid3App::updateCurrentSelection()
 {
+	int numFiles = 0;
 	FileListItem* mp3file = m_view->firstFile();
 	while (mp3file != 0) {
 		if (mp3file->isInSelection()) {
-			updateTags(mp3file->getFile());
+			if (++numFiles > 1) {
+				// we are only interested if 0, 1 or multiple files are selected
+				break;
+			}
 		}
 		mp3file = m_view->nextFile();
 	}
+	if (numFiles > 0) {
+		m_view->frameTableV1()->tableToFrames(numFiles > 1);
+		m_view->frameTableV2()->tableToFrames(numFiles > 1);
+		mp3file = m_view->firstFile();
+		while (mp3file != 0) {
+			if (mp3file->isInSelection()) {
+				TaggedFile* taggedFile = mp3file->getFile();
+				taggedFile->setFramesV1(m_view->frameTableV1()->frames());
+				taggedFile->setFramesV2(m_view->frameTableV2()->frames());
+				if (m_view->isFilenameEditEnabled()) {
+					taggedFile->setFilename(m_view->getFilename());
+				}
+			}
+			mp3file = m_view->nextFile();
+		}
+	}
 	updateModificationState();
 }
 
@@ -2332,19 +2730,7 @@ void Kid3App::updateGuiControls()
 				taggedFile->readTags(false);
 
 #if defined HAVE_ID3LIB && defined HAVE_TAGLIB
-				if (dynamic_cast<Mp3File*>(taggedFile) != 0 &&
-						!taggedFile->isChanged() &&
-						taggedFile->isTagInformationRead() && taggedFile->hasTagV2() &&
-						taggedFile->getTagFormatV2() == QString::null) {
-					TagLibFile* tagLibFile;
-					if ((tagLibFile = new TagLibFile(
-								 taggedFile->getDirInfo(),
-								 taggedFile->getFilename())) != 0) {
-						mp3file->setFile(tagLibFile);
-						taggedFile = tagLibFile;
-						taggedFile->readTags(false);
-					}
-				}
+				taggedFile = readWithTagLibIfId3V24(mp3file, taggedFile);
 #endif
 
 				if (num_files_selected == 0) {
@@ -2377,11 +2763,13 @@ void Kid3App::updateGuiControls()
 		mp3file = m_view->nextFile();
 	}
 
+	TaggedFile::DetailInfo info;
 	if (single_v2_file) {
 		m_framelist->setTags(single_v2_file);
 		m_view->setFilenameEditEnabled(true);
 		m_view->setFilename(single_v2_file->getFilename());
-		m_view->setDetailInfo(single_v2_file->getDetailInfo());
+		single_v2_file->getDetailInfo(info);
+		m_view->setDetailInfo(info);
 		m_view->setTagFormatV1(single_v2_file->getTagFormatV1());
 		m_view->setTagFormatV2(single_v2_file->getTagFormatV2());
 
@@ -2391,7 +2779,7 @@ void Kid3App::updateGuiControls()
 	}
 	else {
 		m_view->setFilenameEditEnabled(false);
-		m_view->setDetailInfo("");
+		m_view->setDetailInfo(info);
 		m_view->setTagFormatV1(QString::null);
 		m_view->setTagFormatV2(QString::null);
 
@@ -2739,8 +3127,10 @@ void Kid3App::editFrame()
 
 /**
  * Delete selected frame.
+ *
+ * @param frameName name of frame to delete, empty to delete selected frame
  */
-void Kid3App::deleteFrame()
+void Kid3App::deleteFrame(const QString& frameName)
 {
 	updateCurrentSelection();
 	TaggedFile* taggedFile = getSelectedFile();
@@ -2760,7 +3150,8 @@ void Kid3App::deleteFrame()
 					firstFile = false;
 					taggedFile = currentFile;
 					m_framelist->setTags(taggedFile);
-					name = m_framelist->getSelectedName();
+					name = frameName.isEmpty() ? m_framelist->getSelectedName() :
+						frameName;
 				}
 				FrameCollection frames;
 				currentFile->getAllFramesV2(frames);
@@ -2781,16 +3172,26 @@ void Kid3App::deleteFrame()
 
 /**
  * Select a frame type and add such a frame to frame list.
+ *
+ * @param frame frame to add, if 0 the user has to select and edit the frame
  */
-void Kid3App::addFrame()
+void Kid3App::addFrame(const Frame* frame)
 {
 	updateCurrentSelection();
 	TaggedFile* taggedFile = getSelectedFile();
-	if (taggedFile &&
-			m_framelist->selectFrame() &&
-			m_framelist->addFrame(true)) {
-		updateAfterFrameModification(taggedFile);
-	} else if (!taggedFile) {
+	if (taggedFile) {
+		if (!frame) {
+			if (m_framelist->selectFrame() &&
+					m_framelist->addFrame(true)) {
+				updateAfterFrameModification(taggedFile);
+			}
+		} else {
+			m_framelist->setFrame(*frame);
+			if (m_framelist->pasteFrame()) {
+				updateAfterFrameModification(taggedFile);
+			}
+		}
+	} else {
 		// multiple files selected
 		FileListItem* mp3file = m_view->firstFile();
 		bool firstFile = true;
@@ -2800,11 +3201,21 @@ void Kid3App::addFrame()
 				if (firstFile) {
 					firstFile = false;
 					taggedFile = mp3file->getFile();
-					if (m_framelist->selectFrame() &&
-							m_framelist->addFrame(true)) {
-						frameId = m_framelist->getSelectedId();
+					m_framelist->setTags(mp3file->getFile());
+					if (!frame) {
+						if (m_framelist->selectFrame() &&
+								m_framelist->addFrame(true)) {
+							frameId = m_framelist->getSelectedId();
+						} else {
+							break;
+						}
 					} else {
-						break;
+						m_framelist->setFrame(*frame);
+						if (m_framelist->pasteFrame()) {
+							frameId = m_framelist->getSelectedId();
+						} else {
+							break;
+						}
 					}
 				} else {
 					m_framelist->setTags(mp3file->getFile());
