@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 9 Jan 2003
  *
- * Copyright (C) 2003-2007  Urs Fleisch
+ * Copyright (C) 2003-2008  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -57,6 +57,7 @@ typedef QMainWindow Kid3AppBaseClass;
 #include "freedbconfig.h"
 #include "discogsconfig.h"
 #include "musicbrainzconfig.h"
+#include "filterconfig.h"
 #include "frame.h"
 
 class KURL;
@@ -67,7 +68,12 @@ class StandardTags;
 class FrameList;
 class ImportDialog;
 class ExportDialog;
+class RenDirDialog;
 class NumberTracksDialog;
+class RenDirDialog;
+class FilterDialog;
+class FileFilter;
+class QImage;
 
 /** Kid3 application */
 class Kid3App : public Kid3AppBaseClass
@@ -88,10 +94,14 @@ public:
 	/**
 	 * Open directory.
 	 *
-	 * @param dir     directory or file path
-	 * @param confirm if true ask if there are unsaved changes
+	 * @param dir       directory or file path
+	 * @param confirm   if true ask if there are unsaved changes
+	 * @param fileCheck if true and dir in not directory, only open directory
+	 *                  if dir is a valid file path
+	 *
+	 * @return true if ok.
 	 */
-	void openDirectory(QString dir, bool confirm = false);
+	bool openDirectory(QString dir, bool confirm = false, bool fileCheck = false);
 
 	/**
 	 * Process change of selection.
@@ -176,6 +186,13 @@ public:
 	 */
 	void openDrop(QString txt);
 
+	/**
+	 * Add picture on drop.
+	 *
+	 * @param image dropped image.
+	 */
+	void dropImage(const QImage& image);
+
 	/**
 	 * Edit selected frame.
 	 */
@@ -183,13 +200,51 @@ public:
 
 	/**
 	 * Delete selected frame.
+	 *
+	 * @param frameName name of frame to delete, empty to delete selected frame
 	 */
-	void deleteFrame();
+	void deleteFrame(const QString& frameName = QString::null);
 
 	/**
 	 * Select a frame type and add such a frame to frame list.
+	 *
+	 * @param frame frame to add, if 0 the user has to select and edit the frame
 	 */
-	void addFrame();
+	void addFrame(const Frame* frame = 0);
+
+	/**
+	 * Set the directory name from the tags.
+	 * The directory must not have modified files.
+	 *
+	 * @param tagMask tag mask (bit 0 for tag 1, bit 1 for tag 2)
+	 * @param format  directory name format
+	 * @param create  true to create, false to rename
+	 * @param errStr  if not 0, a string describing the error is returned here
+	 *
+	 * @return true if ok.
+	 */
+	bool renameDirectory(int tagMask, const QString& format,
+											 bool create, QString* errStr);
+
+	/**
+	 * Number tracks in selected files of directory.
+	 *
+	 * @param nr start number
+	 * @param destV1 true to set numbers in tag 1
+	 * @param destV2 true to set numbers in tag 2
+	 */
+	void numberTracks(int nr, bool destV1, bool destV2);
+
+	/**
+	 * Export.
+	 *
+	 * @param tagNr  tag number (1 or 2)
+	 * @param path   path of file
+	 * @param fmtIdx index of format
+	 *
+	 * @return true if ok.
+	 */
+	bool exportTags(int tagNr, const QString& path, int fmtIdx);
 
 	/**
 	 * Display help for a topic.
@@ -220,6 +275,8 @@ public:
 	static DiscogsConfig s_discogsCfg;
 	/** MusicBrainz configuration */
 	static MusicBrainzConfig s_musicBrainzCfg;
+	/** Filter configuration */
+	static FilterConfig s_filterCfg;
 
 protected:
 	/**
@@ -289,7 +346,7 @@ protected:
 	 */
 	virtual void readProperties(
 #if KDE_VERSION >= 0x035c00
-		KConfigGroup& cfg
+		const KConfigGroup& cfg
 #else
 		KConfig* cfg
 #endif
@@ -418,8 +475,10 @@ public slots:
 
 	/**
 	 * Create playlist.
+	 *
+	 * @return true if ok.
 	 */
-	void slotCreatePlaylist();
+	bool slotCreatePlaylist();
 
 	/**
 	 * Import.
@@ -491,6 +550,11 @@ public slots:
 	 */
 	void slotNumberTracks();
 
+	/**
+	 * Filter.
+	 */
+	void slotFilter();
+
 	/**
 	 * Convert ID3v2.3 to ID3v2.4 tags.
 	 */
@@ -517,13 +581,31 @@ private slots:
 	 */
 	void setExportData(int src);
 
+	/**
+	 * Apply a file filter.
+	 *
+	 * @param fileFilter filter to apply.
+	 */
+	void applyFilter(FileFilter& fileFilter);
+
+	/**
+	 * Schedule actions to rename a directory.
+	 */
+	void scheduleRenameActions();
+
 private:
+	friend class ScriptInterface;
+
 	/**
 	 * Save all changed files.
 	 *
-	 * @return true
+	 * @param updateGui true to update GUI (controls, status, cursor)
+	 * @param errStr    if not 0, the error string is returned here and
+	 *                  no dialog is displayed
+	 *
+	 * @return true if ok.
 	 */
-	bool saveDirectory();
+	bool saveDirectory(bool updateGui = false, QString* errStr = 0);
 
 	/**
 	 * If anything was modified, save after asking user.
@@ -532,13 +614,6 @@ private:
 	 */
 	bool saveModified();
 
-	/**
-	 * Set tags in file to tags in GUI controls.
-	 *
-	 * @param mp3file file
-	 */
-	void updateTags(TaggedFile* mp3file);
-
 	/**
 	 * Update modification state, caption and listbox entries.
 	 */
@@ -585,11 +660,30 @@ private:
 	 */
 	void setupImportDialog();
 
+	/**
+	 * Import tags from the import dialog.
+	 *
+	 * @param destV1 true to set tag 1
+	 * @param destV2 true to set tag 2
+	 */
+	void getTagsFromImportDialog(bool destV1, bool destV2);
+
 	/**
 	 * Execute the import dialog.
 	 */
 	void execImportDialog();
 
+	/**
+	 * Import.
+	 *
+	 * @param tagMask tag mask (bit 0 for tag 1, bit 1 for tag 2)
+	 * @param path    path of file
+	 * @param fmtIdx  index of format
+	 *
+	 * @return true if ok.
+	 */
+	bool importTags(int tagMask, const QString& path, int fmtIdx);
+
 	/**
 	 * Show or hide the ID3V1.1 controls according to the settings and
 	 * set the menu entries appropriately.
@@ -602,10 +696,26 @@ private:
 	 */
 	void updateHideV2();
 
+	/**
+	 * Set filter state.
+	 *
+	 * @param val true if list is filtered
+	 */
+	void setFiltered(bool val) { m_filtered = val; }
+
+	/**
+	 * Check filter state.
+	 *
+	 * @return true if list is filtered.
+	 */
+	bool isFiltered() { return m_filtered; }
+
 	/** GUI with controls, created by Qt Designer */
 	Id3Form* m_view;
 	/** true if any file was modified */
 	bool m_modified;
+	/** true if list is filtered */
+	bool m_filtered;
 	/** Copy buffer */
 	FrameCollection m_copyTags;
 	/** Import dialog */
@@ -614,8 +724,12 @@ private:
 	ImportTrackDataVector m_trackDataList;
 	/** Export dialog */
 	ExportDialog* m_exportDialog;
+	/** Rename directory dialog */
+	RenDirDialog* m_renDirDialog;
 	/** Number tracks dialog */
 	NumberTracksDialog* m_numberTracksDialog;
+	/** Filter dialog */
+	FilterDialog* m_filterDialog;
 	/** Frame list */
 	FrameList* m_framelist;
 
