@@ -30,6 +30,7 @@
 #include "standardtags.h"
 #include "dirinfo.h"
 #include "genres.h"
+#include "pictureframe.h"
 #include <qfile.h>
 #include <qdir.h>
 #if QT_VERSION >= 0x040000
@@ -43,7 +44,10 @@
 #endif
 #include <stdio.h>
 #include <mp4.h>
+#include <cstdlib>
+#include <cstring>
 
+/** MPEG4IP version as 16-bit hex number with major and minor version. */
 #if defined MPEG4IP_MAJOR_VERSION && defined MPEG4IP_MINOR_VERSION
 #define MPEG4IP_MAJOR_MINOR_VERSION ((MPEG4IP_MAJOR_VERSION << 8) | MPEG4IP_MINOR_VERSION)
 #else
@@ -93,7 +97,7 @@ static const struct {
 	{ "aART", Frame::FT_Other },
 	{ "pgap", Frame::FT_Other },
 #endif
-	{ "covr", Frame::FT_Other }
+	{ "covr", Frame::FT_Picture }
 };
 
 /**
@@ -130,7 +134,7 @@ static QString getNameForType(Frame::Type type)
  * @param onlyPredefined if true, FT_Unknown is returned for fields which
  *                       are not predefined, else FT_Other
  *
- * @return type, FT_Other for "cpil", "covr",
+ * @return type, FT_Other for "cpil",
  *         FT_Unknown or FT_Other if not predefined field.
  */
 static Frame::Type getTypeForName(const QString& name,
@@ -461,7 +465,12 @@ bool M4aFile::writeTags(bool force, bool* renamed, bool preserve)
 							handle,
 							reinterpret_cast<u_int8_t*>(const_cast<char*>(value.data())),
 							value.size());
-#if MPEG4IP_MAJOR_MINOR_VERSION >= 0x0105
+// While this works on Debian Etch with libmp4v2-dev 1.5.0.1-0.3 from
+// www.debian-multimedia.org, linking on OpenSUSE 10.3 with
+// libmp4v2-devel-1.5.0.1-6 from packman.links2linux.de fails with
+// undefined reference to MP4SetMetadataGrouping. To avoid this,
+// in the line below, 0x105 is replaced by 0x106.
+#if MPEG4IP_MAJOR_MINOR_VERSION >= 0x0106
 					} else if (name == "\251grp") {
 						setOk = MP4SetMetadataGrouping(handle, str);
 #endif
@@ -582,7 +591,7 @@ void M4aFile::setTextField(const QString& name, const QString& value)
 		QByteArray str = value.QCM_toUtf8();
 		MetadataMap::iterator it = m_metadata.find(name);
 		if (it != m_metadata.end()) {
-			if (*it != str) {
+			if (QString::fromUtf8((*it).data(), (*it).size()) != value) {
 				*it = str;
 				markTag2Changed();
 			}
@@ -823,31 +832,20 @@ QString M4aFile::getFileExtension() const
 /**
  * Get technical detail information.
  *
- * @return string with detail information,
- *         "" if no information available.
+ * @param info the detail information is returned here
  */
-QString M4aFile::getDetailInfo() const
+void M4aFile::getDetailInfo(DetailInfo& info) const
 {
-	QString str;
 	if (m_fileRead && m_fileInfo.valid) {
-		str = "MP4 ";
-		if (m_fileInfo.bitrate > 0 && m_fileInfo.bitrate < 999) {
-			str += QString::number(m_fileInfo.bitrate);
-			str += " kbps ";
-		}
-		if (m_fileInfo.sampleRate > 0) {
-			str += QString::number(m_fileInfo.sampleRate);
-			str += " Hz ";
-		}
-		if (m_fileInfo.channels > 0) {
-			str += QString::number(m_fileInfo.channels);
-			str += " Channels ";
-		}
-		if (m_fileInfo.duration > 0) {
-			str += formatTime(m_fileInfo.duration);
-		}
+		info.valid = true;
+		info.format = "MP4";
+		info.bitrate = m_fileInfo.bitrate;
+		info.sampleRate = m_fileInfo.sampleRate;
+		info.channels = m_fileInfo.channels;
+		info.duration = m_fileInfo.duration;
+	} else {
+		info.valid = false;
 	}
-	return str;
 }
 
 /**
@@ -886,14 +884,15 @@ bool M4aFile::setFrameV2(const Frame& frame)
 	QString name(frame.getName(true));
 	MetadataMap::iterator it = m_metadata.find(name);
 	if (it != m_metadata.end()) {
-		if (name != "covr") {
-			*it = frame.getValue().QCM_toUtf8();
+		if (frame.getType() != Frame::FT_Picture) {
+			QByteArray str = frame.getValue().QCM_toUtf8();
+			if (*it != str) {
+				*it = str;
+				markTag2Changed();
+			}
 		} else {
-			if (!frame.getFieldList().empty()) {
-				const Frame::Field& fld = frame.getFieldList().front();
-				if (fld.m_id == Frame::Field::ID_Data) {
-					*it = fld.m_value.toByteArray();
-				}
+			if (PictureFrame::getData(frame, *it)) {
+				markTag2Changed();
 			}
 		}
 		return true;
@@ -921,16 +920,9 @@ bool M4aFile::addFrameV2(Frame& frame)
 		}
 	}
 	name = frame.getName(true);
-	if (name == "covr") {
-		if (!frame.getFieldList().empty() &&
-				frame.getFieldList().front().m_id == Frame::Field::ID_Data) {
-			m_metadata[name] = frame.getFieldList().front().m_value.toByteArray();
-		} else {
-			Frame::Field coverField;
-			coverField.m_id = Frame::Field::ID_Data;
-			coverField.m_value = QByteArray();
-			frame.fieldList().clear();
-			frame.fieldList().push_back(coverField);
+	if (type == Frame::FT_Picture) {
+		if (!PictureFrame::getData(frame, m_metadata[name])) {
+			PictureFrame::setFields(frame);
 			m_metadata[name] = QByteArray();
 		}
 	} else {
@@ -976,16 +968,12 @@ void M4aFile::getAllFramesV2(FrameCollection& frames)
 			 ++it) {
 		name = it.key();
 		Frame::Type type = getTypeForName(name);
-		if (name != "covr") {
+		if (type != Frame::FT_Picture) {
 			value = QString::fromUtf8((*it).data(), (*it).size());
 			frames.insert(Frame(type, value, name, -1));
 		} else {
-			Frame frame(type, "", name, -1);
-			Frame::Field coverField;
-			coverField.m_id = Frame::Field::ID_Data;
-			coverField.m_value = QByteArray(*it);
-			frame.fieldList().clear();
-			frame.fieldList().push_back(coverField);
+			PictureFrame frame(*it);
+			frame.setInternalName(name);
 			frames.insert(frame);
 		}
 	}
@@ -1010,14 +998,15 @@ QStringList M4aFile::getFrameIds() const
 		Frame::FT_Bpm,
 		Frame::FT_Composer,
 		Frame::FT_Disc,
-		Frame::FT_EncodedBy
+		Frame::FT_EncodedBy,
+		Frame::FT_Picture
 	};
 
 	QStringList lst;
 	for (unsigned i = 0; i < sizeof(types) / sizeof(types[0]); ++i) {
 		lst.append(QCM_translate(Frame::getNameFromType(types[i])));
 	}
-	lst << "covr" << "cpil";
+	lst << "cpil";
 #if MPEG4IP_MAJOR_MINOR_VERSION >= 0x0105
 	lst << "\251grp";
 #endif
