@@ -494,7 +494,8 @@ static bool setYear(ID3_Tag* tag, int num)
 		} else {
 			str = "";
 		}
-		changed = setTextField(tag, ID3FID_YEAR, str);
+		changed = getTextField(tag, ID3FID_YEAR) != str &&
+		          setTextField(tag, ID3FID_YEAR, str);
 	}
 	return changed;
 }
@@ -522,7 +523,8 @@ static bool setTrackNum(ID3_Tag* tag, int num, int numTracks = -1)
 		} else {
 			str = "";
 		}
-		changed = setTextField(tag, ID3FID_TRACKNUM, str);
+		changed = getTextField(tag, ID3FID_TRACKNUM) != str &&
+		          setTextField(tag, ID3FID_TRACKNUM, str);
 	}
 	return changed;
 }
@@ -545,7 +547,8 @@ static bool setGenreNum(ID3_Tag* tag, int num)
 		} else {
 			str = "";
 		}
-		changed = setTextField(tag, ID3FID_CONTENTTYPE, str);
+		changed = getTextField(tag, ID3FID_CONTENTTYPE) != str &&
+		          setTextField(tag, ID3FID_CONTENTTYPE, str);
 	}
 	return changed;
 }
@@ -737,7 +740,8 @@ QString Mp3File::getGenreV2()
  */
 void Mp3File::setTitleV1(const QString& str)
 {
-	if (setTextField(m_tagV1, ID3FID_TITLE, str)) {
+	if (getTextField(m_tagV1, ID3FID_TITLE) != str &&
+			setTextField(m_tagV1, ID3FID_TITLE, str)) {
 		markTag1Changed();
 		QString s = checkTruncation(str, StandardTags::TF_Title);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_TITLE, s);
@@ -751,7 +755,8 @@ void Mp3File::setTitleV1(const QString& str)
  */
 void Mp3File::setArtistV1(const QString& str)
 {
-	if (setTextField(m_tagV1, ID3FID_LEADARTIST, str)) {
+	if (getTextField(m_tagV1, ID3FID_LEADARTIST) != str &&
+			setTextField(m_tagV1, ID3FID_LEADARTIST, str)) {
 		markTag1Changed();
 		QString s = checkTruncation(str, StandardTags::TF_Artist);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_LEADARTIST, s);
@@ -765,7 +770,8 @@ void Mp3File::setArtistV1(const QString& str)
  */
 void Mp3File::setAlbumV1(const QString& str)
 {
-	if (setTextField(m_tagV1, ID3FID_ALBUM, str)) {
+	if (getTextField(m_tagV1, ID3FID_ALBUM) != str &&
+			setTextField(m_tagV1, ID3FID_ALBUM, str)) {
 		markTag1Changed();
 		QString s = checkTruncation(str, StandardTags::TF_Album);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_ALBUM, s);
@@ -779,7 +785,8 @@ void Mp3File::setAlbumV1(const QString& str)
  */
 void Mp3File::setCommentV1(const QString& str)
 {
-	if (setTextField(m_tagV1, ID3FID_COMMENT, str)) {
+	if (getTextField(m_tagV1, ID3FID_COMMENT) != str &&
+			setTextField(m_tagV1, ID3FID_COMMENT, str)) {
 		markTag1Changed();
 		QString s = checkTruncation(str, StandardTags::TF_Comment, 28);
 		if (!s.isNull()) setTextField(m_tagV1, ID3FID_COMMENT, s);
@@ -837,7 +844,8 @@ void Mp3File::setGenreV1(const QString& str)
  */
 void Mp3File::setTitleV2(const QString& str)
 {
-	if (setTextField(m_tagV2, ID3FID_TITLE, str, true)) {
+	if (getTextField(m_tagV2, ID3FID_TITLE) != str &&
+			setTextField(m_tagV2, ID3FID_TITLE, str, true)) {
 		markTag2Changed();
 	}
 }
@@ -849,7 +857,8 @@ void Mp3File::setTitleV2(const QString& str)
  */
 void Mp3File::setArtistV2(const QString& str)
 {
-	if (setTextField(m_tagV2, ID3FID_LEADARTIST, str, true)) {
+	if (getTextField(m_tagV2, ID3FID_LEADARTIST) != str &&
+			setTextField(m_tagV2, ID3FID_LEADARTIST, str, true)) {
 		markTag2Changed();
 	}
 }
@@ -861,7 +870,8 @@ void Mp3File::setArtistV2(const QString& str)
  */
 void Mp3File::setAlbumV2(const QString& str)
 {
-	if (setTextField(m_tagV2, ID3FID_ALBUM, str, true)) {
+	if (getTextField(m_tagV2, ID3FID_ALBUM) != str &&
+			setTextField(m_tagV2, ID3FID_ALBUM, str, true)) {
 		markTag2Changed();
 	}
 }
@@ -873,7 +883,8 @@ void Mp3File::setAlbumV2(const QString& str)
  */
 void Mp3File::setCommentV2(const QString& str)
 {
-	if (setTextField(m_tagV2, ID3FID_COMMENT, str, true)) {
+	if (getTextField(m_tagV2, ID3FID_COMMENT) != str &&
+			setTextField(m_tagV2, ID3FID_COMMENT, str, true)) {
 		markTag2Changed();
 	}
 }
@@ -917,7 +928,8 @@ void Mp3File::setGenreV2(const QString& str)
 				markTag2Changed();
 			}
 		} else {
-			if (setTextField(m_tagV2, ID3FID_CONTENTTYPE, str, true)) {
+			if (getTextField(m_tagV2, ID3FID_CONTENTTYPE) != str &&
+					setTextField(m_tagV2, ID3FID_CONTENTTYPE, str, true)) {
 				markTag2Changed();
 			}
 		}
@@ -971,89 +983,80 @@ bool Mp3File::hasTagV2() const
 /**
  * Get technical detail information.
  *
- * @return string with detail information,
- *         "" if no information available.
+ * @param info the detail information is returned here
  */
-QString Mp3File::getDetailInfo() const {
+void Mp3File::getDetailInfo(DetailInfo& info) const
+{
 	if (getFilename().right(4).QCM_toLower() == ".aac") {
-		return "AAC";
+		info.valid = true;
+		info.format = "AAC";
+		return;
 	}
-	QString str("");
-	const Mp3_Headerinfo* info = NULL;
+
+	const Mp3_Headerinfo* headerInfo = 0;
 	if (m_tagV1) {
-		info = m_tagV1->GetMp3HeaderInfo();
+		headerInfo = m_tagV1->GetMp3HeaderInfo();
 	} else if (m_tagV2) {
-		info = m_tagV2->GetMp3HeaderInfo();
+		headerInfo = m_tagV2->GetMp3HeaderInfo();
 	}
-	if (info) {
-		switch (info->version) {
+	if (headerInfo) {
+		info.valid = true;
+		switch (headerInfo->version) {
 			case MPEGVERSION_1:
-				str.append("MPEG 1 ");
+				info.format = "MPEG 1 ";
 				break;
 			case MPEGVERSION_2:
-				str.append("MPEG 2 ");
+				info.format = "MPEG 2 ";
 				break;
 			case MPEGVERSION_2_5:
-				str.append("MPEG 2.5 ");
+				info.format = "MPEG 2.5 ";
 				break;
 			default:
 				; // nothing
 		}
-		switch (info->layer) {
+		switch (headerInfo->layer) {
 			case MPEGLAYER_I:
-				str.append("Layer 1 ");
+				info.format += "Layer 1";
 				break;
 			case MPEGLAYER_II:
-				str.append("Layer 2 ");
+				info.format += "Layer 2";
 				break;
 			case MPEGLAYER_III:
-				str.append("Layer 3 ");
+				info.format += "Layer 3";
 				break;
 			default:
 				; // nothing
 		}
-		int kb = info->bitrate;
+		info.bitrate = headerInfo->bitrate / 1000;
 #ifndef HAVE_NO_ID3LIB_VBR
-		if (info->vbr_bitrate > 1000) {
-			str.append("VBR ");
-			kb = info->vbr_bitrate;
+		if (headerInfo->vbr_bitrate > 1000) {
+			info.vbr = true;
+			info.bitrate = headerInfo->vbr_bitrate / 1000;
 		}
 #endif
-		if (kb > 1000 && kb < 999000) {
-			kb /= 1000;
-			QString kbStr;
-			kbStr.setNum(kb);
-			kbStr.append(" kbps ");
-			str.append(kbStr);
-		}
-		int hz = info->frequency;
-		if (hz > 0) {
-			QString hzStr;
-			hzStr.setNum(hz);
-			hzStr.append(" Hz ");
-			str.append(hzStr);
-		}
-		switch (info->channelmode) {
+		info.sampleRate = headerInfo->frequency;
+		switch (headerInfo->channelmode) {
 			case MP3CHANNELMODE_STEREO:
-				str.append("Stereo ");
+				info.channelMode = DetailInfo::CM_Stereo;
+				info.channels = 2;
 				break;
 			case MP3CHANNELMODE_JOINT_STEREO:
-				str.append("Joint Stereo ");
+				info.channelMode = DetailInfo::CM_JointStereo;
+				info.channels = 2;
 				break;
 			case MP3CHANNELMODE_DUAL_CHANNEL:
-				str.append("Dual ");
+				info.channels = 2;
 				break;
 			case MP3CHANNELMODE_SINGLE_CHANNEL:
-				str.append("Single ");
+				info.channels = 1;
 				break;
 			default:
 				; // nothing
 		}
-		if (info->time > 0) {
-			str.append(formatTime(info->time));
-		}
+		info.duration = headerInfo->time;
+	} else {
+		info.valid = false;
 	}
-	return str;
 }
 
 /**
@@ -1130,7 +1133,7 @@ static const struct TypeStrOfId {
 } typeStrOfId[] = {
 	{ Frame::FT_UnknownFrame,   0 },                                                                   /* ???? */
 	{ Frame::FT_Other,          I18N_NOOP("AENC - Audio encryption") },                                /* AENC */
-	{ Frame::FT_Other,          I18N_NOOP("APIC - Attached picture") },                                /* APIC */
+	{ Frame::FT_Picture,        I18N_NOOP("APIC - Attached picture") },                                /* APIC */
 	{ Frame::FT_Other,          0 },                                                                   /* ASPI */
 	{ Frame::FT_Comment,        I18N_NOOP("COMM - Comments") },                                        /* COMM */
 	{ Frame::FT_Other,          I18N_NOOP("COMR - Commercial") },                                      /* COMR */
@@ -1223,6 +1226,7 @@ static const struct TypeStrOfId {
 	{ Frame::FT_Other,          I18N_NOOP("WXXX - User defined URL link") }                            /* WXXX */
 };
 
+/** Not instantiated class to check array size at compilation time. */
 class not_used { int array_size_check[
 		sizeof(typeStrOfId) / sizeof(typeStrOfId[0]) == ID3FID_WWWUSER + 1
 		? 1 : -1 ]; };
@@ -1486,14 +1490,21 @@ bool Mp3File::setFrameV2(const Frame& frame)
 								}
 						}
 					}
-					setString(fld, value);
+					if (getString(fld) != value) {
+						setString(fld, value);
+						markTag2Changed();
+					}
 					return true;
 				} else if ((fld = id3Frame->GetField(ID3FN_URL)) != 0) {
-					fld->Set(value.QCM_latin1());
+					if (getString(fld) != value) {
+						fld->Set(value.QCM_latin1());
+						markTag2Changed();
+					}
 					return true;
 				}
 			} else {
 				setId3v2Frame(id3Frame, frame);
+				markTag2Changed();
 				return true;
 			}
 		}
@@ -1519,8 +1530,11 @@ bool Mp3File::addFrameV2(Frame& frame)
 		id = getId3libFrameIdForType(frame.getType());
 	} else {
 		id = getId3libFrameIdForName(frame.getName());
+		if (id == ID3FID_NOFRAME) {
+			id = ID3FID_USERTEXT;
+		}
 	}
-	if (id != ID3FID_NOFRAME && m_tagV2) {
+	if (id != ID3FID_NOFRAME && id != ID3FID_SETSUBTITLE && m_tagV2) {
 		ID3_Frame* id3Frame = new ID3_Frame(id);
 		if (id3Frame) {
 			ID3_Field* fld = id3Frame->GetField(ID3FN_TEXT);
@@ -1532,6 +1546,21 @@ bool Mp3File::addFrameV2(Frame& frame)
 				}
 				fld->SetEncoding(enc);
 			}
+			if (id == ID3FID_USERTEXT && !frame.getName().startsWith("TXXX")) {
+				fld = id3Frame->GetField(ID3FN_DESCRIPTION);
+				if (fld) {
+					setString(fld, frame.getName());
+				}
+			} else if (id == ID3FID_PICTURE) {
+				fld = id3Frame->GetField(ID3FN_MIMETYPE);
+				if (fld) {
+					setString(fld, "image/jpeg");
+				}
+				fld = id3Frame->GetField(ID3FN_PICTURETYPE);
+				if (fld) {
+					fld->Set(ID3PT_COVERFRONT);
+				}
+			}
 			if (!frame.fieldList().empty()) {
 				setId3v2Frame(id3Frame, frame);
 			}
@@ -1640,6 +1669,21 @@ void Mp3File::getAllFramesV2(FrameCollection& frames)
 			getTypeStringForId3libFrameId(id3Frame->GetID(), type, name);
 			Frame frame(type, "", name, i++);
 			frame.setValue(getFieldsFromId3Frame(id3Frame, frame.fieldList()));
+			if (id3Frame->GetID() == ID3FID_USERTEXT ||
+					id3Frame->GetID() == ID3FID_WWWUSER) {
+				const Frame::FieldList& fields = frame.getFieldList();
+				for (Frame::FieldList::const_iterator it = fields.begin();
+						 it != fields.end();
+						 ++it) {
+					if ((*it).m_id == Frame::Field::ID_Description) {
+						QString description = (*it).m_value.toString();
+						if (!description.isEmpty()) {
+							frame.setInternalName(QString(name) + '\n' + description);
+						}
+						break;
+					}
+				}
+			}
 			frames.insert(frame);
 		}
 #ifdef WIN32
@@ -1659,7 +1703,13 @@ void Mp3File::getAllFramesV2(FrameCollection& frames)
  */
 QStringList Mp3File::getFrameIds() const
 {
-	QStringList lst(TaggedFile::getFrameIds());
+	QStringList lst;
+	for (int type = Frame::FT_FirstFrame; type <= Frame::FT_LastFrame; ++type) {
+		if (type != Frame::FT_Part) {
+			lst.append(QCM_translate(Frame::getNameFromType(
+																 static_cast<Frame::Type>(type))));
+		}
+	}
 	for (int i = 0; i <= ID3FID_WWWUSER; ++i) {
 		if (typeStrOfId[i].type == Frame::FT_Other) {
 			const char* s = typeStrOfId[i].str;
