@@ -261,6 +261,7 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"ORIGINALDATE",    // FT_OriginalDate,
 		"PART",            // FT_Part,
 		"PERFORMER",       // FT_Performer,
+		"UNKNOWN",         // FT_Picture,
 		"PUBLISHER",       // FT_Publisher,
 		"SUBTITLE",        // FT_Subtitle,
 		"WEBSITE",         // FT_Website,
@@ -581,22 +582,20 @@ QString OggFile::getFileExtension() const
 /**
  * Get technical detail information.
  *
- * @return string with detail information,
- *         "" if no information available.
+ * @param info the detail information is returned here
  */
-QString OggFile::getDetailInfo() const
+void OggFile::getDetailInfo(DetailInfo& info) const
 {
-	QString str;
 	if (m_fileRead && m_fileInfo.valid) {
-		str = QString("Ogg Vorbis %1 kbps %2 Hz %3 Channels ").
-			arg(m_fileInfo.bitrate / 1000).
-			arg(m_fileInfo.sampleRate).
-			arg(m_fileInfo.channels);
-		if (m_fileInfo.duration > 0) {
-			str += formatTime(m_fileInfo.duration);
-		}
+		info.valid = true;
+		info.format = "Ogg Vorbis";
+		info.bitrate = m_fileInfo.bitrate / 1000;
+		info.sampleRate = m_fileInfo.sampleRate;
+		info.channels = m_fileInfo.channels;
+		info.duration = m_fileInfo.duration;
+	} else {
+		info.valid = false;
 	}
-	return str;
 }
 
 /**
@@ -635,20 +634,31 @@ bool OggFile::setFrameV2(const Frame& frame)
 	if (frame.getType() == Frame::FT_Track) {
 		int numTracks = getTotalNumberOfTracksIfEnabled();
 		if (numTracks > 0) {
-			setTextField("TRACKTOTAL", QString::number(numTracks));
+			QString numTracksStr = QString::number(numTracks);
+			if (getTextField("TRACKTOTAL") != numTracksStr) {
+				setTextField("TRACKTOTAL", numTracksStr);
+				markTag2Changed();
+			}
 		}
 	}
 
 	// If the frame has an index, change that specific frame
 	int index = frame.getIndex();
 	if (index != -1 && index < static_cast<int>(m_comments.size())) {
+		QString value = frame.getValue();
 #if QT_VERSION >= 0x040000
-		m_comments[index].setValue(frame.getValue());
+		if (m_comments[index].getValue() != value) {
+			m_comments[index].setValue(value);
+			markTag2Changed();
+		}
 		return true;
 #else
 		CommentList::iterator it = m_comments.at(index);
 		if (it != m_comments.end()) {
-			(*it).setValue(frame.getValue());
+			if ((*it).getValue() != value) {
+				(*it).setValue(value);
+				markTag2Changed();
+			}
 			return true;
 		}
 #endif
@@ -761,7 +771,12 @@ QStringList OggFile::getFrameIds() const
 		"VOLUME"
 	};
 
-	QStringList lst(TaggedFile::getFrameIds());
+	QStringList lst;
+	for (int k = Frame::FT_FirstFrame; k <= Frame::FT_LastFrame; ++k) {
+		if (k != Frame::FT_Picture) {
+			lst.append(QCM_translate(Frame::getNameFromType(static_cast<Frame::Type>(k))));
+		}
+	}
 	for (unsigned i = 0; i < sizeof(fieldNames) / sizeof(fieldNames[0]); ++i) {
 		lst.append(fieldNames[i]);
 	}
@@ -809,7 +824,7 @@ bool OggFile::FileInfo::read(const char* fn)
 	return valid;
 }
 #else // HAVE_VORBIS
-QString OggFile::getDetailInfo() const { return ""; }
+void OggFile::getDetailInfo(DetailInfo& info) const { info.valid = false; }
 unsigned OggFile::getDuration() const { return 0; }
 #endif // HAVE_VORBIS
 
