@@ -62,28 +62,25 @@
 #include <taglib/attachedpictureframe.h>
 #include <taglib/uniquefileidentifierframe.h>
 
-#ifdef TAGLIB_SUPPORTS_GEOB_FRAMES
-#include <taglib/generalencapsulatedobjectframe.h>
-#else
+#if TAGLIB_VERSION <= 0x010400
 #include "taglibext/generalencapsulatedobjectframe.h"
-#endif
-#ifdef TAGLIB_SUPPORTS_URLLINK_FRAMES
-#include <taglib/urllinkframe.h>
-#else
 #include "taglibext/urllinkframe.h"
-#endif
-#ifdef TAGLIB_SUPPORTS_USLT_FRAMES
-#include <taglib/unsynchronizedlyricsframe.h>
-#else
 #include "taglibext/unsynchronizedlyricsframe.h"
-#endif
-
 #include "taglibext/speex/speexfile.h"
 #include "taglibext/speex/taglib_speexfiletyperesolver.h"
 #include "taglibext/trueaudio/ttafile.h"
 #include "taglibext/trueaudio/taglib_trueaudiofiletyperesolver.h"
 #include "taglibext/wavpack/wvfile.h"
 #include "taglibext/wavpack/taglib_wavpackfiletyperesolver.h"
+#else
+#include <taglib/generalencapsulatedobjectframe.h>
+#include <taglib/urllinkframe.h>
+#include <taglib/unsynchronizedlyricsframe.h>
+#include <taglib/speexfile.h>
+#include <taglib/trueaudiofile.h>
+#include <taglib/wavpackfile.h>
+#endif
+
 #include "taglibext/aac/aacfiletyperesolver.h"
 #include "taglibext/mp2/mp2filetyperesolver.h"
 
@@ -115,10 +112,20 @@ TagLibFile::~TagLibFile()
  */
 void TagLibFile::readTags(bool force)
 {
-	QCM_QCString fn = QFile::encodeName(getDirInfo()->getDirname() + QDir::separator() + currentFilename());
+	QString fileName = getDirInfo()->getDirname() + QDir::separator() + currentFilename();
+	QCM_QCString fn = QFile::encodeName(fileName);
 
 	if (force || m_fileRef.isNull()) {
+#if TAGLIB_VERSION > 0x010400 && defined _WIN32
+		int fnLen = fileName.length();
+		wchar_t* fnWs = new wchar_t[fnLen + 1];
+		fnWs[fnLen] = 0;
+		fileName.toWCharArray(fnWs);
+		m_fileRef = TagLib::FileRef(TagLib::FileName(fnWs));
+		delete [] fnWs;
+#else
 		m_fileRef = TagLib::FileRef(fn);
+#endif
 		m_tagV1 = 0;
 		m_tagV2 = 0;
 		markTag1Changed(false);
@@ -134,7 +141,7 @@ void TagLibFile::readTags(bool force)
 		TagLib::MPC::File* mpcFile;
 		TagLib::WavPack::File* wvFile;
 #endif
-		TagLib::TTA::File* ttaFile;
+		TagLib::TrueAudio::File* ttaFile;
 		if ((mpegFile = dynamic_cast<TagLib::MPEG::File*>(file)) != 0) {
 			if (!m_tagV1) {
 				m_tagV1 = mpegFile->ID3v1Tag();
@@ -173,7 +180,7 @@ void TagLibFile::readTags(bool force)
 				markTag2Changed(false);
 			}
 #endif
-		} else if ((ttaFile = dynamic_cast<TagLib::TTA::File*>(file)) != 0) {
+		} else if ((ttaFile = dynamic_cast<TagLib::TrueAudio::File*>(file)) != 0) {
 			if (!m_tagV1) {
 				m_tagV1 = ttaFile->ID3v1Tag();
 				markTag1Changed(false);
@@ -267,7 +274,7 @@ bool TagLibFile::writeTags(bool force, bool* renamed, bool preserve)
 		} else {
 			if ((m_tagV2 && (force || isTag2Changed())) ||
 					(m_tagV1 && (force || isTag1Changed()))) {
-				TagLib::TTA::File* ttaFile = dynamic_cast<TagLib::TTA::File*>(file);
+				TagLib::TrueAudio::File* ttaFile = dynamic_cast<TagLib::TrueAudio::File*>(file);
 #ifndef MPC_ID3V1
 				// it does not work if there is also an ID3 tag (bug in TagLib?)
 				TagLib::MPC::File* mpcFile = dynamic_cast<TagLib::MPC::File*>(file);
@@ -276,19 +283,19 @@ bool TagLibFile::writeTags(bool force, bool* renamed, bool preserve)
 					mpcFile->remove(TagLib::MPC::File::ID3v1 | TagLib::MPC::File::ID3v2);
 					fileChanged = true;
 				} else if (wvFile) {
-					wvFile->remove(TagLib::WavPack::File::ID3v1);
+					wvFile->strip(TagLib::WavPack::File::ID3v1);
 					fileChanged = true;
 				} else
 #endif
 				if (ttaFile) {
 					if (m_tagV1 && (force || isTag1Changed()) && m_tagV1->isEmpty()) {
-						ttaFile->remove(TagLib::TTA::File::ID3v1);
+						ttaFile->strip(TagLib::TrueAudio::File::ID3v1);
 						fileChanged = true;
 						markTag1Changed(false);
 						m_tagV1 = 0;
 					}
 					if (m_tagV2 && (force || isTag2Changed()) && m_tagV2->isEmpty()) {
-						ttaFile->remove(TagLib::TTA::File::ID3v2);
+						ttaFile->strip(TagLib::TrueAudio::File::ID3v2);
 						fileChanged = true;
 						markTag2Changed(false);
 						m_tagV2 = 0;
@@ -630,7 +637,7 @@ bool TagLibFile::makeTagV1Settable()
 			TagLib::MPC::File* mpcFile;
 			TagLib::WavPack::File* wvFile;
 #endif
-			TagLib::TTA::File* ttaFile;
+			TagLib::TrueAudio::File* ttaFile;
 			if ((mpegFile = dynamic_cast<TagLib::MPEG::File*>(file)) != 0) {
 				m_tagV1 = mpegFile->ID3v1Tag(true);
 			} else if ((flacFile = dynamic_cast<TagLib::FLAC::File*>(file)) != 0) {
@@ -641,7 +648,7 @@ bool TagLibFile::makeTagV1Settable()
 			} else if ((wvFile = dynamic_cast<TagLib::WavPack::File*>(file)) != 0) {
 				m_tagV1 = wvFile->ID3v1Tag(true);
 #endif
-			} else if ((ttaFile = dynamic_cast<TagLib::TTA::File*>(file)) != 0) {
+			} else if ((ttaFile = dynamic_cast<TagLib::TrueAudio::File*>(file)) != 0) {
 				m_tagV1 = ttaFile->ID3v1Tag(true);
 			}
 		}
@@ -663,7 +670,7 @@ bool TagLibFile::makeTagV2Settable()
 			TagLib::FLAC::File* flacFile;
 			TagLib::MPC::File* mpcFile;
 			TagLib::WavPack::File* wvFile;
-			TagLib::TTA::File* ttaFile;
+			TagLib::TrueAudio::File* ttaFile;
 			if ((mpegFile = dynamic_cast<TagLib::MPEG::File*>(file)) != 0) {
 				m_tagV2 = mpegFile->ID3v2Tag(true);
 			} else if ((flacFile = dynamic_cast<TagLib::FLAC::File*>(file)) != 0) {
@@ -672,7 +679,7 @@ bool TagLibFile::makeTagV2Settable()
 				m_tagV2 = mpcFile->APETag(true);
 			} else if ((wvFile = dynamic_cast<TagLib::WavPack::File*>(file)) != 0) {
 				m_tagV2 = wvFile->APETag(true);
-			} else if ((ttaFile = dynamic_cast<TagLib::TTA::File*>(file)) != 0) {
+			} else if ((ttaFile = dynamic_cast<TagLib::TrueAudio::File*>(file)) != 0) {
 				m_tagV2 = ttaFile->ID3v2Tag(true);
 			}
 		}
@@ -1120,7 +1127,7 @@ bool TagLibFile::isTagV1Supported() const
 	return (!m_fileRef.isNull() && (file = m_fileRef.file()) != 0 &&
 					(dynamic_cast<TagLib::MPEG::File*>(file) != 0 ||
 					 dynamic_cast<TagLib::FLAC::File*>(file) != 0 ||
-					 dynamic_cast<TagLib::TTA::File*>(file) != 0
+					 dynamic_cast<TagLib::TrueAudio::File*>(file) != 0
 #ifdef MPC_ID3V1
 					 || dynamic_cast<TagLib::MPC::File*>(file)  != 0
 					 || dynamic_cast<TagLib::WavPack::File*>(file) != 0
@@ -1142,113 +1149,97 @@ bool TagLibFile::hasTagV2() const
 /**
  * Get technical detail information.
  *
- * @return string with detail information,
- *         "" if no information available.
+ * @param info the detail information is returned here
  */
-QString TagLibFile::getDetailInfo() const {
-	QString str;
+void TagLibFile::getDetailInfo(DetailInfo& info) const
+{
 	TagLib::AudioProperties* audioProperties;
 	if (!m_fileRef.isNull() &&
 			(audioProperties = m_fileRef.audioProperties()) != 0) {
-		const char* channelModeStr = 0;
 		TagLib::MPEG::Properties* mpegProperties;
 		TagLib::Vorbis::Properties* oggProperties;
 		TagLib::FLAC::Properties* flacProperties;
 		TagLib::MPC::Properties* mpcProperties;
-		TagLib::Speex::Properties* speexProperties;
-		TagLib::TTA::Properties* ttaProperties;
+		TagLib::Ogg::Speex::Properties* speexProperties;
+		TagLib::TrueAudio::Properties* ttaProperties;
 		TagLib::WavPack::Properties* wvProperties;
+		info.valid = true;
 		if ((mpegProperties =
 				 dynamic_cast<TagLib::MPEG::Properties*>(audioProperties)) != 0) {
 			if (getFilename().right(4).QCM_toLower() == ".aac") {
-				return "AAC";
+				info.format = "AAC";
+				return;
 			}
 			switch (mpegProperties->version()) {
 				case TagLib::MPEG::Header::Version1:
-					str += "MPEG 1 ";
+					info.format = "MPEG 1 ";
 					break;
 				case TagLib::MPEG::Header::Version2:
-					str += "MPEG 2 ";
+					info.format = "MPEG 2 ";
 					break;
 				case TagLib::MPEG::Header::Version2_5:
-					str += "MPEG 2.5 ";
+					info.format = "MPEG 2.5 ";
 					break;
-					//! @todo is there information about VBR.
 			}
 			int layer = mpegProperties->layer();
 			if (layer >= 1 && layer <= 3) {
-				str += "Layer ";
-				str += QString::number(layer);
-				str += ' ';
+				info.format += "Layer ";
+				info.format += QString::number(layer);
 			}
 			switch (mpegProperties->channelMode()) {
 				case TagLib::MPEG::Header::Stereo:
-					channelModeStr = "Stereo ";
+					info.channelMode = DetailInfo::CM_Stereo;
+					info.channels = 2;
 					break;
 				case TagLib::MPEG::Header::JointStereo:
-					channelModeStr = "Joint Stereo ";
+					info.channelMode = DetailInfo::CM_JointStereo;
+					info.channels = 2;
 					break;
 				case TagLib::MPEG::Header::DualChannel:
-					channelModeStr = "Dual ";
+					info.channels = 2;
 					break;
 				case TagLib::MPEG::Header::SingleChannel:
-					channelModeStr = "Single ";
+					info.channels = 1;
 					break;
 			}
 		} else if ((oggProperties =
 								dynamic_cast<TagLib::Vorbis::Properties*>(audioProperties)) !=
 							 0) {
-			str += "Ogg Vorbis ";
+			info.format = "Ogg Vorbis";
 		} else if ((flacProperties =
 								dynamic_cast<TagLib::FLAC::Properties*>(audioProperties)) !=
 							 0) {
-			str += "FLAC ";
+			info.format = "FLAC";
 		} else if ((mpcProperties =
 								dynamic_cast<TagLib::MPC::Properties*>(audioProperties)) != 0) {
-			str += "MPC ";
+			info.format = "MPC";
 		} else if ((speexProperties =
-								dynamic_cast<TagLib::Speex::Properties*>(audioProperties)) != 0) {
-			str += QString("Speex %1 ").arg(speexProperties->speexVersion());
+								dynamic_cast<TagLib::Ogg::Speex::Properties*>(audioProperties)) != 0) {
+			info.format = QString("Speex %1").arg(speexProperties->speexVersion());
 		} else if ((ttaProperties =
-								dynamic_cast<TagLib::TTA::Properties*>(audioProperties)) != 0) {
-			str += "True Audio ";
-			str += QString::number(ttaProperties->ttaVersion());
-			str += " ";
-			str += QString::number(ttaProperties->bitsPerSample());
-			str += " bit ";
+								dynamic_cast<TagLib::TrueAudio::Properties*>(audioProperties)) != 0) {
+			info.format = "True Audio ";
+			info.format += QString::number(ttaProperties->ttaVersion());
+			info.format += " ";
+			info.format += QString::number(ttaProperties->bitsPerSample());
+			info.format += " bit";
 		} else if ((wvProperties =
 								dynamic_cast<TagLib::WavPack::Properties*>(audioProperties)) != 0) {
-			str += "WavPack ";
-			str += QString::number(wvProperties->version(), 16);
-			str += " ";
-			str += QString::number(wvProperties->bitsPerSample());
-			str += " bit ";
+			info.format = "WavPack ";
+			info.format += QString::number(wvProperties->version(), 16);
+			info.format += " ";
+			info.format += QString::number(wvProperties->bitsPerSample());
+			info.format += " bit";
 		}
-		int bitrate = audioProperties->bitrate();
-		if (bitrate > 0 && bitrate < 999) {
-			str += QString::number(bitrate);
-			str += " kbps ";
-		}
-		int sampleRate = audioProperties->sampleRate();
-		if (sampleRate > 0) {
-			str += QString::number(sampleRate);
-			str += " Hz ";
-		}
-		if (channelModeStr) {
-			str += channelModeStr;
-		} else {
-			int channels = audioProperties->channels();
-			if (channels > 0) {
-				str += QString::number(channels);
-				str += " Channels ";
-			}
-		}
-		int length = audioProperties->length();
-		if (length > 0) {
-			str += formatTime(length);
+		info.bitrate = audioProperties->bitrate();
+		info.sampleRate = audioProperties->sampleRate();
+		if (audioProperties->channels() > 0) {
+			info.channels = audioProperties->channels();
 		}
+		info.duration = audioProperties->length();
+	} else {
+		info.valid = false;
 	}
-	return str;
 }
 
 /**
@@ -1285,11 +1276,11 @@ QString TagLibFile::getFileExtension() const
 			return ".flac";
 		} else if (dynamic_cast<TagLib::MPC::File*>(file) != 0) {
 			return ".mpc";
-		} else if (dynamic_cast<TagLib::Speex::File*>(file) != 0) {
+		} else if (dynamic_cast<TagLib::Ogg::Speex::File*>(file) != 0) {
 			return ".spx";
 		} else if (dynamic_cast<TagLib::WavPack::File*>(file) != 0) {
 			return ".wv";
-		} else if (dynamic_cast<TagLib::TTA::File*>(file) != 0) {
+		} else if (dynamic_cast<TagLib::TrueAudio::File*>(file) != 0) {
 			return ".tta";
 		}
 	}
@@ -1371,7 +1362,7 @@ static const struct TypeStrOfId {
 	bool supported;
 } typeStrOfId[] = {
 	{ Frame::FT_Other,          I18N_NOOP("AENC - Audio encryption"), false },
-	{ Frame::FT_Other,          I18N_NOOP("APIC - Attached picture"), true },
+	{ Frame::FT_Picture,        I18N_NOOP("APIC - Attached picture"), true },
 	{ Frame::FT_Other,          I18N_NOOP("ASPI - Audio seek point index"), false },
 	{ Frame::FT_Comment,        I18N_NOOP("COMM - Comments"), true },
 	{ Frame::FT_Other,          I18N_NOOP("COMR - Commercial"), false },
@@ -2126,6 +2117,7 @@ void setValue(TagLib::ID3v2::AttachedPictureFrame* f, const TagLib::String& text
 /**
  * Set the fields in a TagLib ID3v2 frame.
  *
+ * @param self   this TagLibFile instance
  * @param tFrame TagLib frame to set
  * @param frame  frame with field values
  */
@@ -2307,19 +2299,39 @@ bool TagLibFile::setFrameV2(const Frame& frame)
 				// However when removing the old frame and adding a new frame,
 				// the indices of all frames get invalid.
 				setId3v2Frame(frameList[index], frame);
+				markTag2Changed();
 				return true;
 			}
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
 			TagLib::String key = QSTRING_TO_TSTRING(frame.getName(true));
 			TagLib::String value = QSTRING_TO_TSTRING(frame.getValue());
-#ifdef TAGLIB_XIPHCOMMENT_REMOVEFIELD_CRASHES
+#if TAGLIB_VERSION <= 0x010400
+			// Remove all fields with that key, because TagLib <= 1.4 crashes
+			// using an invalidated iterator after calling erase().
 			oggTag->addField(key, value, true);
 #else
-			// This will crash because TagLib uses an invalidated iterator
-			// after calling erase(). I hope this will be fixed in the next
-			// version. Until then, remove all fields with that key.
-			oggTag->removeField(key, oldValue);
-			oggTag->addField(key, value, false);
+			const TagLib::Ogg::FieldListMap& fieldListMap = oggTag->fieldListMap();
+			if (fieldListMap.contains(key) && fieldListMap[key].size() > 1) {
+				int i = 0;
+				TagLib::String oldValue(TagLib::String::null);
+				for (TagLib::Ogg::FieldListMap::ConstIterator it = fieldListMap.begin();
+						 it != fieldListMap.end();
+						 ++it) {
+					TagLib::StringList stringList = (*it).second;
+					for (TagLib::StringList::ConstIterator slit = stringList.begin();
+							 slit != stringList.end();
+							 ++slit) {
+						if (i++ == index) {
+							oldValue = *slit;
+							break;
+						}
+					}
+				}
+				oggTag->removeField(key, oldValue);
+				oggTag->addField(key, value, false);
+			} else {
+				oggTag->addField(key, value, true);
+			}
 #endif
 			if (frame.getType() == Frame::FT_Track) {
 				int numTracks = getTotalNumberOfTracksIfEnabled();
@@ -2327,10 +2339,12 @@ bool TagLibFile::setFrameV2(const Frame& frame)
 					oggTag->addField("TRACKTOTAL", TagLib::String::number(numTracks), true);
 				}
 			}
+			markTag2Changed();
 			return true;
 		} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
 			apeTag->addValue(QSTRING_TO_TSTRING(frame.getName(true)),
 											 QSTRING_TO_TSTRING(frame.getValue()));
+			markTag2Changed();
 			return true;
 		}
 	}
@@ -2373,6 +2387,7 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"ORIGINALDATE",    // FT_OriginalDate,
 		"PART",            // FT_Part,
 		"PERFORMER",       // FT_Performer,
+		"UNKNOWN",         // FT_Picture,
 		"PUBLISHER",       // FT_Publisher,
 		"SUBTITLE",        // FT_Subtitle,
 		"WEBSITE",         // FT_Website,
@@ -2472,6 +2487,21 @@ static QString getApeName(const Frame& frame)
 	}
 }
 
+/**
+ * Check if an ID3v2.4.0 frame ID is valid.
+ *
+ * @param frameId frame ID (4 characters)
+ *
+ * @return true if frame ID is valid.
+ */
+static bool isFrameIdValid(const QString& frameId)
+{
+	Frame::Type type;
+	const char* str;
+	getTypeStringForFrameId(TagLib::ByteVector(frameId.QCM_latin1(), 4), type, str);
+	return type != Frame::FT_UnknownFrame;
+}
+
 /**
  * Add a frame in the tags 2.
  *
@@ -2497,7 +2527,7 @@ bool TagLibFile::addFrameV2(Frame& frame)
 			if (frameId.startsWith("T")) {
 				if (frameId == "TXXX") {
 					id3Frame = new TagLib::ID3v2::UserTextIdentificationFrame(enc);
-				} else {
+				} else if (isFrameIdValid(frameId)) {
 					id3Frame = new TagLib::ID3v2::TextIdentificationFrame(
 						TagLib::ByteVector(frameId.QCM_latin1(), frameId.length()), enc);
 					id3Frame->setText(""); // is necessary for createFrame() to work
@@ -2508,6 +2538,10 @@ bool TagLibFile::addFrameV2(Frame& frame)
 				id3Frame = new TagLib::ID3v2::AttachedPictureFrame;
 				if (id3Frame) {
 					((TagLib::ID3v2::AttachedPictureFrame*)id3Frame)->setTextEncoding(enc);
+					((TagLib::ID3v2::AttachedPictureFrame*)id3Frame)->setMimeType(
+						"image/jpeg");
+					((TagLib::ID3v2::AttachedPictureFrame*)id3Frame)->setType(
+						TagLib::ID3v2::AttachedPictureFrame::FrontCover);
 				}
 			} else if (frameId == "UFID") {
 				// the bytevector must not be empty
@@ -2521,7 +2555,7 @@ bool TagLibFile::addFrameV2(Frame& frame)
 			} else if (frameId.startsWith("W")) {
 				if (frameId == "WXXX") {
 					id3Frame = new TagLib::ID3v2::UserUrlLinkFrame(enc);
-				} else {
+				} else if (isFrameIdValid(frameId)) {
 					id3Frame = new TagLib::ID3v2::UrlLinkFrame(
 						TagLib::ByteVector(frameId.QCM_latin1(), frameId.length()));
 					id3Frame->setText("http://"); // is necessary for createFrame() to work
@@ -2529,6 +2563,15 @@ bool TagLibFile::addFrameV2(Frame& frame)
 			} else if (frameId == "USLT") {
 				id3Frame = new TagLib::ID3v2::UnsynchronizedLyricsFrame(enc);
 			}
+			if (!id3Frame) {
+				TagLib::ID3v2::UserTextIdentificationFrame* txxxFrame =
+					new TagLib::ID3v2::UserTextIdentificationFrame(enc);
+				txxxFrame->setDescription(QSTRING_TO_TSTRING(frame.getName()));
+				id3Frame = txxxFrame;
+				frame.setInternalName("TXXX - User defined text information");
+			} else {
+				frame.setInternalName(name);
+			}
 			if (id3Frame) {
 				if (!frame.fieldList().empty()) {
 					frame.setValueFromFieldList();
@@ -2544,7 +2587,6 @@ bool TagLibFile::addFrameV2(Frame& frame)
 #else
 				id3v2Tag->addFrame(id3Frame);
 #endif
-				frame.setInternalName(name);
 				frame.setIndex(id3v2Tag->frameList().size() - 1);
 				if (frame.fieldList().empty()) {
 					// add field list to frame
@@ -2560,7 +2602,15 @@ bool TagLibFile::addFrameV2(Frame& frame)
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
 			QString name(getVorbisName(frame));
 			TagLib::String tname = QSTRING_TO_TSTRING(name);
-			oggTag->addField(tname, QSTRING_TO_TSTRING(frame.getValue()));
+			TagLib::String tvalue = QSTRING_TO_TSTRING(frame.getValue());
+			if (tvalue.isEmpty()) {
+				tvalue = " "; // empty values are not added by TagLib
+			}
+#if TAGLIB_VERSION <= 0x010400
+			oggTag->addField(tname, tvalue);
+#else
+			oggTag->addField(tname, tvalue, false);
+#endif
 			frame.setInternalName(name);
 
 			const TagLib::Ogg::FieldListMap& fieldListMap = oggTag->fieldListMap();
@@ -2574,7 +2624,7 @@ bool TagLibFile::addFrameV2(Frame& frame)
 					found = true;
 					break;
 				}
-				++index;
+				index += (*it).second.size();
 			}
 			frame.setIndex(found ? index : -1);
 			markTag2Changed();
@@ -2636,12 +2686,11 @@ bool TagLibFile::deleteFrameV2(const Frame& frame)
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
 			TagLib::String key =
 				QSTRING_TO_TSTRING(frame.getName(true));
-#ifdef TAGLIB_XIPHCOMMENT_REMOVEFIELD_CRASHES
+#if TAGLIB_VERSION <= 0x010400
+			// Remove all fields with that key, because TagLib <= 1.4 crashes
+			// using an invalidated iterator after calling erase().
 			oggTag->removeField(key);
 #else
-			// This will crash because TagLib uses an invalidated iterator
-			// after calling erase(). I hope this will be fixed in the next
-			// version. Until then, remove all fields with that key.
 			oggTag->removeField(key, QSTRING_TO_TSTRING(frame.getValue()));
 #endif
 			markTag2Changed();
@@ -2765,6 +2814,24 @@ void TagLibFile::getAllFramesV2(FrameCollection& frames)
 				getTypeStringForFrameId((*it)->frameID(), type, name);
 				Frame frame(type, TStringToQString((*it)->toString()), name, i++);
 				frame.setValue(getFieldsFromId3Frame(*it, frame.fieldList(), type));
+				if ((*it)->frameID().mid(1, 3) == "XXX") {
+					const Frame::FieldList& fields = frame.getFieldList();
+					for (Frame::FieldList::const_iterator it = fields.begin();
+							 it != fields.end();
+							 ++it) {
+						if ((*it).m_id == Frame::Field::ID_Description) {
+							QString description = (*it).m_value.toString();
+							if (!description.isEmpty()) {
+								if (description.startsWith("QuodLibet::")) {
+									// remove ExFalso/QuodLibet "namespace"
+									description = description.mid(11);
+								}
+								frame.setInternalName(QString(name) + '\n' + description);
+							}
+							break;
+						}
+					}
+				}
 				frames.insert(frame);
 			}
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
@@ -2810,9 +2877,14 @@ void TagLibFile::getAllFramesV2(FrameCollection& frames)
  */
 QStringList TagLibFile::getFrameIds() const
 {
-	QStringList lst(TaggedFile::getFrameIds());
-	TagLib::ID3v2::Tag* id3v2Tag;
-	if ((id3v2Tag = dynamic_cast<TagLib::ID3v2::Tag*>(m_tagV2)) != 0) {
+	TagLib::ID3v2::Tag* id3v2Tag = dynamic_cast<TagLib::ID3v2::Tag*>(m_tagV2);
+	QStringList lst;
+	for (int k = Frame::FT_FirstFrame; k <= Frame::FT_LastFrame; ++k) {
+		if (id3v2Tag || k != Frame::FT_Picture) {
+			lst.append(QCM_translate(Frame::getNameFromType(static_cast<Frame::Type>(k))));
+		}
+	}
+	if (id3v2Tag) {
 		for (unsigned i = 0; i < sizeof(typeStrOfId) / sizeof(typeStrOfId[0]); ++i) {
 			const TypeStrOfId& ts = typeStrOfId[i];
 			if (ts.type == Frame::FT_Other && ts.supported && ts.str) {
@@ -2864,9 +2936,11 @@ QStringList TagLibFile::getFrameIds() const
  */
 void TagLibFile::staticInit()
 {
+#if TAGLIB_VERSION <= 0x010400
 	TagLib::FileRef::addFileTypeResolver(new SpeexFileTypeResolver);
 	TagLib::FileRef::addFileTypeResolver(new WavPackFileTypeResolver);
 	TagLib::FileRef::addFileTypeResolver(new TTAFileTypeResolver);
+#endif
 	TagLib::FileRef::addFileTypeResolver(new AACFileTypeResolver);
 	TagLib::FileRef::addFileTypeResolver(new MP2FileTypeResolver);
 }
