@@ -41,15 +41,7 @@
 #define TAGLIB_VERSION (((TAGLIB_MAJOR_VERSION) << 16) + \
 												((TAGLIB_MINOR_VERSION) << 8) + (TAGLIB_PATCH_VERSION))
 
-#if (TAGLIB_VERSION <= 0x010400)
-/** Defined if TagLib::Ogg::XiphComment::removeField() crashes */
-#define TAGLIB_XIPHCOMMENT_REMOVEFIELD_CRASHES
-//#else
-//#define TAGLIB_SUPPORTS_GEOB_FRAMES
-//#define TAGLIB_SUPPORTS_URLLINK_FRAMES
-//#define TAGLIB_SUPPORTS_USLT_FRAMES
-#endif
-
+/** Convert QString @a s to a TagLib::String. */
 #if QT_VERSION >= 0x040000
 #define QSTRING_TO_TSTRING(s) TagLib::String(s.toUtf8().data(), TagLib::String::UTF8)
 #else
@@ -387,10 +379,9 @@ public:
 	/**
 	 * Get technical detail information.
 	 *
-	 * @return string with detail information,
-	 *         "" if no information available.
+	 * @param info the detail information is returned here
 	 */
-	virtual QString getDetailInfo() const;
+	virtual void getDetailInfo(DetailInfo& info) const;
 
 	/**
 	 * Get duration of file.
